package Engine::Actions::AdminDbFileCheck;
use strict;

use XFileConfig;
use Engine::Core::Action( 'IMPLEMENTS' => [qw(get_db_files get_fs_files del_from_db del_from_fs)] );
use XUtils;
use LWP::UserAgent;
use HTTP::Request;
use JSON;

sub main
{
   my $filter = "AND srv_id IN ($1)" if $f->{srv_id} =~ /^([0-9,]+)$/;
   my $servers = $db->SelectARef("SELECT srv_id, srv_name FROM Servers WHERE srv_status != 'OFF' $filter");

   my %srv_files_count;
   my $files_count;
   for(@{ $db->SelectARef("SELECT srv_id, COUNT(*) AS files_count FROM Files WHERE 1 $filter GROUP BY srv_id") }) {
      $srv_files_count{int($_->{srv_id})} = int($_->{files_count});
      $files_count += $_->{files_count};
   }

   return $ses->PrintTemplate('admin_db_file_check.html',
      servers => JSON::encode_json($servers),
      srv_files_count => JSON::encode_json(\%srv_files_count),
      files_count => $files_count,
      srv_ids => JSON::encode_json([ map { int($_->{srv_id}) } @$servers ]),
      servers => JSON::encode_json({ map { (int($_->{srv_id}) => { srv_name => $_->{srv_name} }) } @$servers }),
   );
}

sub get_db_files {
   return $ses->PrintJSON({ status => 'ERROR', msg => 'No srv_id specified' }) if !$f->{srv_id};
   
   my $filter = "AND srv_id=" . int($f->{srv_id});
   
   $|=1;
   iimy $sth = $db->master()->dbh()->prepare("SELECT f.usr_id, u.usr_login, u.usr_premium_expire > NOW() AS premium, srv_id, file_id, file_code, file_real, file_name, file_size, DATE(file_created) AS file_date, file_last_download, file_downloads
      FROM Files f
      LEFT JOIN Users u ON u.usr_id=f.usr_id
      WHERE 1 $filter");
   $sth->execute();
   
   print "X-Accel-Buffering: no\n";
   print "Content-Encoding: none\n";
   print "X-Items-Count: ", $db->SelectOne("SELECT COUNT(file_id) FROM Files WHERE 1 $filter"), "\n";
   print "Content-type: application/json\n\n";
   
   STDOUT->flush();
   
   while(my $row = $sth->fetchrow_hashref())
   {
      $row->{download_link} = $ses->makeFileLink($row);
      $row->{file_size} = $ses->makeFileSize($row->{file_size});
      $row->{utype} = $row->{usr_id} ? ($row->{premium} ? "prem" : "reg") : "anon";
      print JSON::encode_json($row), "\n";
      STDOUT->flush();
   }

   print JSON::encode_json({ done => 1 }), "\n";
   return;
}

sub get_fs_files
{
   return $ses->PrintJSON({ status => 'ERROR', msg => 'No srv_id specified' }) if !$f->{srv_id};

   my $server = $db->SelectRow("SELECT * FROM Servers WHERE srv_id=?", $f->{srv_id}) || die("No such file server");
   my $req = HTTP::Request->new(POST => "$server->{srv_cgi_url}/api.cgi", [], "op=get_files_list2&fs_key=$server->{srv_key}&r=123");

   print "X-Accel-Buffering: no\n";
   print "Content-Encoding: none\n";
   print "X-Items-Count: ", $db->SelectOne("SELECT COUNT(DISTINCT(file_real)) FROM Files WHERE srv_id=?", $server->{srv_id}), "\n";
   print "Content-type: application/json\n\n";

   $|++;

   my $ua = LWP::UserAgent->new(agent => "XFS-FSServer", timeout => 55);
   my $res = $ua->request($req, sub {
      my ($buffer, $res) = @_;
      print $buffer;
      STDOUT->flush();
   });

   print JSON::encode_json({ error => $res->decoded_content() }) if $res->code() != 200;
   return;
}

sub del_from_db
{
   my $ids = join("','", map { /(\w{12})/ } @{ XUtils::ARef( $f->{'ids[]'} ) });

   my $files = $db->SelectARef("SELECT * FROM Files WHERE file_real IN ('$ids')");
   $db->Exec("DELETE FROM Files WHERE file_real IN ('$ids')");

   return $ses->PrintJSON({ deleted => int(@$files) });
}

sub del_from_fs
{
   return $ses->SendJSON({ error => 'No srv_id specified' }) if !$f->{srv_id};

   my @ids = map { /(\d{5}\/\w{12})$/ } @{ XUtils::ARef( $f->{'ids[]'} ) };
   my $res = $ses->api2($f->{srv_id}, { op => 'delete_files2', list => join("\n", @ids) });
   return $ses->PrintJSON({ status => 'OK', res => $res });
}

1;
