package Engine::Actions::ForgotPass;
use strict;
use Digest::SHA qw(sha256_hex);

use XFileConfig;
use Engine::Core::Action(
   'IMPLEMENTS' => [ qw(send_password_recovery_email reset_password reset_password_submit) ],
   'ANTICSRF_WHITELIST' => [ qw(reset_password) ]
);

sub main {
   $ses->setCaptchaMode( $c->{captcha_mode} || 2 );
   return $ses->redirect( $c->{site_url} ) if $ses->getUser;

   $f->{usr_login} = $ses->SecureStr( $f->{usr_login} );
   my %secure = $ses->SecSave( 0, 0 );
   $ses->PrintTemplate( "forgot_pass.html", %{$f}, %secure, );
}

sub send_password_recovery_email {
   $ses->setCaptchaMode( $c->{captcha_mode} || 2 );
   return main() if !$ses->SecCheck($f->{'rand'}, 0, $f->{code});

   my $user = $db->SelectRow("SELECT * FROM Users WHERE (usr_login=? OR usr_email=?)", ($f->{usr_login}) x 2);
   return $ses->message( $ses->{lang}->{lang_no_login_email} ) unless $user;

   my $code = $ses->randchar(16, 'a-zA-Z0-9');
   my $sign = sha256_hex($code, $c->{license_key});
   my $link = "$c->{site_url}/forgot_pass?reset_password=$code&sign=$sign";
   $db->Exec("INSERT INTO SecurityTokens SET usr_id=?, ip=?, purpose='reset_pass', value=?",
      $user->{usr_id}, $ses->getIP(), $code);

   $ses->require("MailSender")->sendMail(
      $user->{usr_email}, $c->{email_from},
      "$c->{site_name}: password recovery",
      "Please follow this link to continue recovery:\n<a href=\"$link\">$link</a>"
   );

   return $ses->PrintTemplate(
      "message.html",
      err_title => "Notice",
      msg       => "Password recovery link sent to your e-mail"
   );
}

sub reset_password {
   return $ses->message("Wrong signature") if $f->{sign} ne sha256_hex($f->{reset_password}, $c->{license_key});
   $ses->setCaptchaMode( $c->{captcha_mode} || 2 );
   my %secure = $ses->SecSave( 0, 0 );
   return $ses->PrintTemplate("reset_password.html",
      %secure,
      reset_password_code => $f->{reset_password},
      sign => $f->{sign},
   );
}

sub reset_password_submit {
   $ses->setCaptchaMode( $c->{captcha_mode} || 2 );
   return reset_password() if !$ses->SecCheck($f->{'rand'}, 0, $f->{code});
   return $ses->message("Wrong signature") if $f->{sign} ne sha256_hex($f->{reset_password_code}, $c->{license_key});

   my $token = $db->SelectRow("SELECT * FROM SecurityTokens
      WHERE purpose='reset_pass'
      AND value=?
      AND created > NOW() - INTERVAL 1 HOUR",
      $f->{reset_password_code});

   return $ses->message("Security token is not existing or expired") if !$token;

   my $user = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $token->{usr_id});
   return $ses->message("No such user") if !$user;

   $db->Exec( "DELETE FROM Sessions WHERE usr_id=?", $user->{usr_id} );
   my $sess_id = $ses->require("SessionTracker")->StartSession( $user->{usr_id} );
   $ses->setAuthCookie($sess_id);

   $db->Exec( "UPDATE Users SET usr_password='', usr_security_lock='' WHERE usr_id=?", $user->{usr_id} );
   $db->Exec( "UPDATE Users SET usr_status='OK' WHERE usr_id=?", $user->{usr_id} ) if $user->{usr_status} eq 'PENDING';
   return $ses->redirect( $c->{site_url} );
}

1;
