package Engine::Actions::G2faCheck;
use strict;

use XFileConfig;
use Engine::Core::Action ( 'IMPLEMENTS' => [qw(submit)] );

use Auth::GoogleAuth;
use JSON;

sub main
{
   my $ticket = _get_ticket($f->{ticket_id}) || return $ses->redirect("?op=login");

   return $ses->PrintTemplate("g2fa_check.html",
      ticket_id => $ticket->{value},
      domain => $ses->getDomain($c->{site_url})
   );
}

sub submit
{
   my $ticket = _get_ticket($f->{ticket_id}) || return $ses->redirect("?op=login");
   my $sess_id = JSON::decode_json($ticket->{data})->{sess_id};
   my $session = $db->SelectRow("SELECT * FROM Sessions WHERE session_id=?", $sess_id);
   my $user = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $session->{usr_id});

   my $auth = Auth::GoogleAuth->new;

   if(!$auth->verify($f->{code6}, 10, $user->{usr_g2fa_secret}, time(), 30)) {
      $f->{msg} = "Wrong 6-digit-code. Please check that your phone time is in sync.";
      return main();
   }

   $ses->setAuthCookie($sess_id);
   return $ses->redirect("?op=my_account");
}

sub _get_ticket {
   my ($ticket_id) = @_;
   return $db->SelectRow("SELECT * FROM SecurityTokens WHERE purpose='g2fa' AND value=? AND created > NOW() - INTERVAL 30 MINUTE", $ticket_id);
}

1;
