package Engine::Actions::PaymentComplete;
use strict;

use XFileConfig;
use Engine::Core::Action;

use URI::Escape;

sub main
{
   my ($str) = $ENV{QUERY_STRING}=~/payment_complete=(.+)/;
   my ( $id, $usr_id ) = split( /-/, uri_unescape($str) );
   ( $id, $usr_id ) = split( /-/, $ses->getCookie('transaction_id') ) if !$id;
   my $trans = $db->SelectRow(
      "SELECT *, ip as ip, (UNIX_TIMESTAMP()-UNIX_TIMESTAMP(created)) as dt
                               FROM Transactions 
                               WHERE id=?", $id
   ) if $id;
   return $ses->message($ses->{lang}->{lang_no_transaction}) unless $trans;
   return $ses->message($ses->{lang}->{lang_internal_error}) unless $trans->{ip} eq $ses->getIP;
   return $ses->message($ses->{lang}->{lang_account_created_sucessfully})
     if $trans->{dt} > 3600;
   return $ses->message($ses->{lang}->{lang_payment_not_verified})
     unless $trans->{verified};

   my $user = $db->SelectRow(
      "SELECT *, UNIX_TIMESTAMP(usr_premium_expire)-UNIX_TIMESTAMP() as exp_sec 
                              FROM Users 
                              WHERE usr_id=?", $trans->{usr_id}
   );
   return $ses->message($ses->{lang}->{lang_no_such_user}) if !$user;

   if($trans->{target} =~ /^file_(\d+)_access$/)
   {
      my $file = $db->SelectRow("SELECT * FROM Files WHERE file_id=?", $1);
      return $ses->message($ses->{lang}->{lang_no_such_file}) if !$file;

      my $never_logged_in = $db->SelectOne("SELECT usr_lastlogin=0 FROM Users WHERE usr_id=?", $trans->{usr_id});
      if(!$ses->getUser && $never_logged_in)
      {
         my $sess_id = $ses->require("SessionTracker")->StartSession($trans->{usr_id});
         $ses->setAuthCookie($sess_id);
      }

      return $ses->redirect($ses->makeFileLink($file));
   }

   require Time::Elapsed;
   my $et  = new Time::Elapsed;
   my $exp = $et->convert( $user->{exp_sec} );
   $ses->PrintTemplate(
      'message.html',
      err_title => 'Payment completed',
      msg =>
"Your payment processed successfully!<br>You should receive your password on e-mail in few minutes.<br><br>Login: $user->{usr_login}<br>Password: ******<br><br>Your premium account expires in:<br>$exp",
   );
}

1;
