package Engine::Actions::Register;
use strict;
use XUtils;

use XFileConfig;
use Engine::Core::Action ( 'IMPLEMENTS' => [qw(save)] );
use JSON;
use HCE_MD5;

sub main
{
   my ($msg) = @_;
   $f->{msg} ||= $msg;

   return $ses->redirect( $c->{site_url} ) if $ses->getUser;
   return $ses->message($ses->{lang}->{lang_registration_disabled}) if !$c->{reg_enabled};
   my $msg = shift;
   $ses->setCaptchaMode( $c->{captcha_mode} || 2 );
   my %secure = $ses->SecSave( 0, 0 );
   $f->{usr_login} = $ses->SecureStr( $f->{usr_login} );
   $f->{usr_email} = $ses->SecureStr( $f->{usr_email} );
   if ( $f->{aff_id} =~ /^(\d+)$/i )
   {
      $ses->setCookie( "aff", $1, '+14d' );
   }
   my @payout_list = map { { name => $_ } } split( /\s*\,\s*/, $c->{payout_systems} );
   $ses->PrintTemplate(
      "registration.html",
      %secure,
      %{$c},
      'usr_login'                    => $f->{usr_login},
      'usr_email'                    => $f->{usr_email},
      'usr_password'                 => $f->{usr_password},
      'usr_password2'                => $f->{usr_password2},
      'usr_phone'                    => $f->{usr_phone},
      'coupons'                      => $c->{coupons},
      'coupon_code'                  => $f->{coupon_code} || $f->{coupon},
      'usr_pay_email'                => $f->{usr_pay_email},
      "pay_type_$f->{usr_pay_type}"  => 1,
      'msg'                          => $f->{msg} || $msg,
      'paypal_email'                 => $c->{paypal_email},
      'alertpay_email'               => $c->{alertpay_email},
      'webmoney_merchant_id'         => $c->{webmoney_merchant_id},
      'next'                         => $f->{'next'},
      'payout_list'                  => \@payout_list,
      'mod_sec_require_phone_number' => $c->{mod_sec_require_phone_number},
   );
}

sub save
{
   $ses->setCaptchaMode( $c->{captcha_mode} || 2 );
   return $ses->redirect("Registration disabled") if !$c->{reg_enabled};
   return &main() if !$ses->SecCheck( $f->{'rand'}, 0, $f->{code} );
   return &main("Error: $ses->{lang}->{lang_login_too_short}") if length( $f->{usr_login} ) < 4;
   return &main("Error: $ses->{lang}->{lang_login_too_long}")  if length( $f->{usr_login} ) > 32;
   return &main("Error: Invalid login: reserved word")         if $f->{usr_login} =~ /^(admin|images|captchas|files)$/;
   return &main("Error: $ses->{lang}->{lang_invalid_login}") unless $f->{usr_login} =~ /^[\w\-\_]+$/;
   return &main("Error: Password contain bad symbols") if $f->{usr_password} =~ /[<>"]/;
   return &main("Error: $ses->{lang}->{lang_pass_too_short}")  if length( $f->{usr_password} ) < 4;
   return &main("Error: $ses->{lang}->{lang_pass_too_long}")   if length( $f->{usr_password} ) > 32;
   return &main("Error: $ses->{lang}->{lang_pass_dont_match}") if $f->{usr_password} ne $f->{usr_password2};
   return &main("Error: $ses->{lang}->{lang_invalid_email}")
     unless $f->{usr_email} =~ /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/;
   return &main("Error: $ses->{lang}->{lang_mailhost_banned}")
     if $c->{mailhosts_not_allowed} && $f->{usr_email} =~ /\@$c->{mailhosts_not_allowed}/i;
   return &main("Error: $ses->{lang}->{lang_login_exist}")
     if $db->SelectOne( "SELECT usr_id FROM Users WHERE usr_login=?", $f->{usr_login} );
   return &main("Error: $ses->{lang}->{lang_email_exist}")
     if $db->SelectOne( "SELECT usr_id FROM Users WHERE usr_email=?", $f->{usr_email} );

   my $phone = $f->{usr_phone};
   $phone =~ s/\D//g;
   return &main("Error: you haven't specified your phone number") if $c->{mod_sec_require_phone_number} && !$f->{usr_phone};
   return &main("Error: invalid phone number") if $f->{usr_phone} && $f->{usr_phone} !~ /^[0-9+-]+$/;
   return &main("Error: phone number exists")
     if $f->{usr_phone} && $db->SelectOne( "SELECT usr_id FROM Users WHERE usr_phone=?", $phone );

   my $confirm_key = $ses->randchar(6) if $c->{mod_sec_require_phone_number} || $c->{registration_confirm_email};
   my $premium_days =  _parse_coupons()->{lc($f->{coupon_code})} || 0;

   my $usersRegistry = $ses->require("UsersRegistry");

   my $usr_id = $usersRegistry->createUser({
      (map { $_ => $f->{"usr_$_"} } qw(login password email pay_email pay_type)),
      premium_days => $premium_days,
      security_lock => $confirm_key,
      status => $confirm_key ? 'PENDING' : 'OK',
      profit_mode => $c->{m_y_default} ? $c->{m_y_default} : 'PPD',
      phone => $phone,
      aff_id => $ses->getCookie('aff') || 0,
   });

   if($f->{next})
   {
      my $hce = HCE_MD5->new($c->{dl_key}, "XFileSharingPRO");
      my $next = $hce->hce_block_decrypt($ses->decode32($f->{next}));
      my $req = eval { decode_json($next) };
   
      my $query = join('&', map { $_.'='.$req->{$_} } keys(%$req)) if $req;
      $usersRegistry->setDelayedRedirect($usr_id => "?$query") if $query;
   }

   $db->Exec("INSERT INTO Stats SET day=CURDATE(), registered=1 ON DUPLICATE KEY UPDATE registered=registered+1");

   if ( $c->{mod_sec_require_phone_number} )
   {
      # Registration requires a different approach to security token handling rather than XUtils::SMSConfirm()
      my $secret_code = $ses->randchar(8);
      $db->Exec("INSERT INTO SecurityTokens SET usr_id=?, purpose='registration', ip=?, value=?, phone=?",
         $usr_id, $ses->getIP, $secret_code, $phone);
      return $ses->message("Error while sending SMS: $ses->{errstr}")
         if !$ses->SendSMS( $phone, "$c->{site_name} registration confirmation code: $secret_code" );
      return $ses->redirect("$c->{site_url}/confirm_phone?usr_id=$usr_id&purpose=registration");
   }
   elsif ($c->{registration_confirm_email})
   {
      my $t = $ses->CreateTemplate("registration_email.html");
      $t->param( 'usr_login' => $f->{usr_login}, 'usr_password' => $f->{usr_password}, 'confirm_id' => "$usr_id-$confirm_key" );
      $c->{email_text} = 1;
      $ses->require("MailSender")->sendMail( $f->{usr_email}, $c->{email_from}, "$c->{site_name} registration confirmation", $t->output );
      return $ses->PrintTemplate(
         'message.html',
         err_title => 'Account created',
         msg       => $ses->{lang}->{lang_account_created}
      );
   }

   $f->{login}    = $f->{usr_login};
   $f->{password} = $f->{usr_password};

   my $sess_id = $ses->require("SessionTracker")->StartSession( $usr_id );
   $ses->setAuthCookie($sess_id);

   my $user = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $usr_id);
   return if $usersRegistry->checkForDelayedRedirects($user);
   return $ses->redirect( "$c->{site_url}/account/" );
}

sub _parse_coupons
{
   my $hh = {};
   for ( split( /\|/, $c->{coupons} ) )
   {
      $hh->{ lc($1) } = $2 if /^(.+?)=(\d+)$/;
   }

   return $hh;
}

1;
