package Engine::Actions::ReportFile;
use strict;

use XFileConfig;
use Engine::Core::Action( 'IMPLEMENTS' => [qw(send)] );

sub main
{
   my $file = $db->SelectRow( "SELECT * FROM Files WHERE file_code=?", $f->{id} );
   return $ses->message($ses->{lang}->{lang_no_such_file}) unless $file;
   my %secure = $ses->SecSave( 2, 5 );
   $f->{$_} = $ses->SecureStr( $f->{$_} ) for keys %$f;
   $f->{email} ||= $ses->getUser->{usr_email} if $ses->getUser;
   $ses->PrintTemplate(
      "report_file.html",
      %{$f},
      %secure,
      'file_name' => $file->{file_name},
      'ip'        => $ses->getIP(),
   );
}

sub send
{
   return &main() unless $ses->getEnv('REQUEST_METHOD') eq 'POST';
   return &main() unless $ses->SecCheck( $f->{'rand'}, 2, $f->{code} );

   my $file = $db->SelectRow( "SELECT * FROM Files WHERE file_code=?", $f->{id} );
   return $ses->message("No such file") unless $file;

   $f->{msg} .= "Email is not valid. " unless $f->{email} =~ /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/;
   $f->{msg} .= "Name required. " unless $f->{name};
   $f->{msg} .= "Message required. " unless $f->{message};

   return &ReportFile if $f->{msg};

   #$f->{message}="Reason: $f->{reason}\n\n$f->{message}";
   $f->{$_} = $ses->SecureStr( $f->{$_} ) for keys %$f;

   my $ip = $ses->getIP;

   $db->Exec(
"INSERT INTO Reports SET file_id=?, usr_id=?, filename=?, name=?, email=?, reason=?, info=?, ip=?, status='PENDING', created=NOW()",
      $file->{file_id}, $file->{usr_id}, $file->{file_name}, $f->{name}, $f->{email}, $f->{reason}, $f->{message},
      $ses->getIP() );
   $f->{subject} = "$c->{site_name}: File reported";
   $f->{message} =
"File was reported on $c->{site_name}.\n\nFilename: $file->{file_name}\n\nName: $f->{name}\nE-mail: $f->{email}\nReason: $f->{reason}\nIP: $ip\n\n$f->{message}";
   $c->{email_text} = 1;
   $ses->require("MailSender")->sendMail( $c->{contact_email}, $c->{email_from}, $f->{subject}, $f->{message} );
   return $ses->redirect("$c->{site_url}/?msg=Report sent successfully");
}

1;
