package Engine::Actions::UploadForm;
use strict;

use XFileConfig;
use Engine::Core::Action;

use XUtils;

sub main
{
   return $ses->message( $c->{maintenance_upload_msg} || "Uploads are temporarily disabled due to site maintenance",
      "Site maintenance" )
     if $c->{maintenance_upload};
   return $ses->redirect("?op=login") if !$c->{enabled_anon} && !$ses->getUser;
   $ses->redirect("?op=login") if !$c->{upload_on} && $ses->{utype} eq 'anon';

   my $approved_affiliate = 1 if $ses->getUser() && $ses->getUser()->{usr_aff_enabled};
   return $ses->message( "Uploads are disabled for your user type", "Upload error" ) if !$c->{upload_on} && !$approved_affiliate;

   if ( $c->{upload_disabled_countries} && -f "$c->{cgi_path}/GeoLite2-Country.mmdb" )
   {
      require Geo::IP2;
      my $gi      = Geo::IP2->new("$c->{cgi_path}/GeoLite2-Country.mmdb");
      my $country = $gi->country_code_by_addr( $ses->getIP );
      for ( split( /\s*\,\s*/, $c->{upload_disabled_countries} ) )
      {
         return $ses->message("Uploads are disabled for your country: $country") if $_ eq $country;
      }
   }

   my $server = $ses->require("ServersRegistry")->selectServer($ses->{user});
   my $server_torrent = $ses->require("ServersRegistry")->selectServer( $ses->getUser(), "srv_torrent=1" );
   $server = $db->SelectRow( "SELECT * FROM Servers WHERE srv_id=?", $f->{srv_id} )
     if $ses->getUser && $ses->getUser->{usr_adm} && $f->{srv_id};

   return $ses->redirect('?op=admin_server_add') if !$server && $ses->getUser && $ses->getUser->{usr_adm};
   return $ses->message(
      "We're sorry, there are no servers available for upload at the moment.<br>Refresh this page in some minutes.")
     unless $server;
   $server->{srv_htdocs_url} =~ s/\/(\w+)$//;
   $server->{srv_tmp_url} = "$server->{srv_htdocs_url}/tmp";
   $server_torrent->{srv_htdocs_url} =~ s/\/(\w+)$//;
   $server_torrent->{srv_tmp_url} = "$server_torrent->{srv_htdocs_url}/tmp";
   my @url_fields = map { { 'number' => $_, 'enable_file_descr' => $c->{enable_file_descr} } } ( 1 .. $c->{max_upload_files} );
   my @folders_tree = $ses->require("FldUtils")->buildFoldersTree( usr_id => $ses->getUserId );

   my $stats;
   if ( $c->{show_server_stats} )
   {
      $stats =
        $db->SelectRow("SELECT SUM(srv_files) as files_total, ROUND(SUM(srv_disk)/1073741824,2) as used_total FROM Servers");
      $stats->{user_total} = $db->SelectOne("SELECT COUNT(*) FROM Users");
   }

   my $mmrr = $c->{"\x6d\x5f\x72"};
   my ( $leech_on, $leech_left_mb );
   if ( $c->{max_rs_leech} && $ses->getUser )
   {
      $leech_left_mb =
        ( $ses->getUser->{usr_max_rs_leech} || $c->{max_rs_leech} ) -
        $db->SelectOne(
         "SELECT ROUND(SUM(size)/1048576) FROM IP2RS WHERE created>NOW()-INTERVAL 24 HOUR AND (usr_id=? OR ip=?)",
         $ses->getUserId, $ses->getIP );
      $leech_left_mb = 0 if $leech_left_mb < 0;
      $leech_on      = 1 if $c->{remote_url};     #$leech_left_mb>0 &&
   }

   my $mmtt = $ses->iPlg('t');
   my $mmtt_on = $mmtt && $c->{"\x74\x6f\x72\x72\x65\x6e\x74\x5f\x64\x6c"};

   #$mmtt=0 unless $server_torrent->{srv_id};
   my $tt_msg;
   if ( $mmtt && !$server_torrent->{srv_id} )
   {
      $mmtt_on = 0;
      $tt_msg .= $ses->{lang}->{lang_no_torrent_srv} . "<br>";
   }
   if (  $mmtt
      && $c->{torrent_dl_slots}
      && $db->SelectOne( "SELECT COUNT(*) FROM Torrents WHERE usr_id=? AND status='WORKING'", $ses->getUserId ) >=
      $c->{torrent_dl_slots} )
   {
      $mmtt_on = 0;
      $tt_msg .= $ses->{lang}->{lang_full_torr_slots} . " ($c->{torrent_dl_slots})<br>";
   }

   my $mmff    = $ses->iPlg('f');
   my $mmff_on = $mmff && $c->{flash_upload};
   my $exts    = join ';', map { "*.$_" } split( /\|/, $c->{ext_allowed} );
   my $exts2   = join ':', map { "*.$_" } split( /\|/, $c->{ext_allowed} );
   $exts2 ||= '*.*';

   my $udata = $ses->getUserData;
   my $sites = $ses->require("PluginsHelper")->getPluginsOptions( 'Leech', $udata ) if $leech_on;
   my @supported;
   for (@$sites)
   {
      push @supported, $_->{domain} if $udata->{ $_->{name} } || $c->{ $_->{name} };
   }

   my $supported_sites = join ', ', sort @supported;

   my $data = $db->SelectARef( "SELECT name,value FROM UserData WHERE usr_id=?", $ses->getUserId );
   my @site_logins =
     map { { name => $_->{name}, value => $_->{value} } } grep { $_->{name} =~ /_logins$/i && $_->{value} } @$data;

   my $ftp_servers = $db->SelectARef("SELECT * FROM Servers WHERE srv_ftp AND srv_status!='OFF' ORDER BY srv_id")
     if $c->{ftp_mod} && $ses->{utype} ne 'anon';
   my $i = 1;
   for (@$ftp_servers)
   {
      $_->{name}      = "FTP" . ( $i++ );
      $_->{usr_login} = $ses->getUser->{usr_login};
      $_->{host}      = $1 if $_->{srv_htdocs_url} =~ /^https?:\/\/([^:\/]+)/;
   }

   my $usr_id = $ses->getUserId if $ses->getUser;

   my $file_upload_url = "$server->{srv_cgi_url}/upload.cgi?upload_type=file&utype=$ses->{utype}";

   my $domain = $1 if $c->{site_url} =~ /^https?:\/\/([^\/]+)/;
   $domain =~ s/^www\.//;

   $ses->ApplyHooks('upload_form', $server);

   my $ftp_password = $ses->require("UsersRegistry")->genFTPPassword($ses->getUser()) if $c->{ftp_passwords_mode} eq 'random';

   $ses->PrintTemplate(
      "upload_form.html",
      'ext_allowed'               => $c->{ext_allowed},
      'ext_not_allowed'           => $c->{ext_not_allowed},
      'max_upload_files'          => $c->{max_upload_files},
      'max_upload_files_rows'     => $c->{max_upload_files} <= 10 ? $c->{max_upload_files} : 10,
      'max_upload_filesize'       => $c->{max_upload_filesize},
      'max_upload_filesize_bytes' => $c->{max_upload_filesize} * 1024 * 1024,
      'enable_file_descr'         => $c->{enable_file_descr},
      'remote_url'                => $c->{remote_url},
      'sanitize_filename'         => $c->{sanitize_filename},
      'add_filename_postfix'      => $c->{add_filename_postfix},
      'm_i_adult'                 => $c->{m_i_adult},
      'ftp_upload_user'           => $c->{ftp_upload},
      'domain'                    => $domain,

      'srv_cgi_url'    => $server->{srv_cgi_url},
      'srv_tmp_url'    => $server->{srv_tmp_url},
      'srv_htdocs_url' => $server->{srv_htdocs_url},

      'srv_torrent_cgi_url' => $server_torrent->{srv_cgi_url},
      'srv_torrent_tmp_url' => $server_torrent->{srv_tmp_url},

      'ftp_servers' => $ftp_servers,

      'sess_id'         => $ses->getCookie( $ses->{auth_cook} ),
      'folders_tree'    => \@folders_tree,
      'mmrr'            => $mmrr,
      'mmtt'            => $mmtt,
      'mmtt_on'         => $mmtt_on,
      'tt_msg'          => $tt_msg,
      'mmff'            => $mmff,
      'mmff_on'         => $mmff_on,
      'utype'           => $ses->{utype},
      'url_fields'      => \@url_fields,
      'supported_sites' => $supported_sites,
      'exts'            => $exts,
      'exts2'           => $exts2,
      'leech_left_mb'   => $leech_left_mb,
      'leech_on'        => $leech_on,
      %{$stats},
      'site_logins'                   => \@site_logins,
      'max_rs_leech'                  => $c->{max_rs_leech},
      'rnd'                           => $ses->randchar(6),
      "link_format_$c->{link_format}" => 1,
      "users_total"                   => $db->SelectOne("SELECT COUNT(*) FROM Users"),
      'file_public_default'           => $c->{file_public_default} || 0,
      'agree_tos_default'             => $c->{agree_tos_default} || 0,
      'file_upload_url'               => $file_upload_url,
      'ftp_upload'                    => $c->{ftp_upload},
      'token_my_files'                => $ses->genToken(op => 'my_files'),
      'm_n_chunked_upload'            => ($ses->iPlg("n") && $c->{m_n_chunked_upload})||0,
      "remote_upload_mode_$c->{remote_upload_mode}" => 1,
      "ftp_passwords_mode_$c->{ftp_passwords_mode}" => 1,
      "ftp_password"    => $ftp_password,
   );
}

1;
