package Engine::Actions::UploadResult;
use strict;

use XFileConfig;
use Engine::Core::Action;

use XUtils;

sub main
{
   my $fnames = XUtils::ARef( $f->{'fn'} );
   my $status = XUtils::ARef( $f->{'st'} );

   my @arr;
   return if $c->{site_url} !~ /\/\/(www\.|)$ses->{dc}/i || !$ses->{dc};

   my %features;
   for ( my $i = 0 ; $i <= $#$fnames ; $i++ )
   {
      $fnames->[$i] = $ses->SecureStr( $fnames->[$i] );
      $status->[$i] = $ses->SecureStr( $status->[$i] );
      unless ( $status->[$i] eq 'OK' )
      {
         push @arr, { file_name => $fnames->[$i], 'error' => " $status->[$i]" };
         next;
      }
      my $file = $db->SelectRow(
         "SELECT f.*, s.srv_htdocs_url
                                 FROM Files f, Servers s
                                 WHERE f.file_code=?
                                 AND f.srv_id=s.srv_id
                                 AND f.file_created > NOW()-INTERVAL 15 MINUTE", $fnames->[$i]
      );
      next unless $file;
      $file->{file_size2}    = $file->{file_size};
      $file->{file_size}     = $ses->makeFileSize( $file->{file_size} );
      $file->{download_link} = $ses->makeFileLink($file);
      $file->{delete_link}   = "$file->{download_link}?killcode=$file->{file_del_id}";
      if ( $c->{m_i} && $file->{file_name} =~ /\.(jpg|jpeg|gif|png|bmp)$/i )
      {
         $features{has_image_url} = 1 if $c->{m_i_hotlink_orig};
         $ses->getThumbLink($file);
      }
      if ( $c->{m_v} && $c->{video_embed} && $file->{file_spec} =~ /^V/ )
      {
         my @fields =
           qw(vid vid_length vid_width vid_height vid_bitrate vid_audio_bitrate vid_audio_rate vid_codec vid_audio_codec vid_fps);
         my @vinfo = split( /\|/, $file->{file_spec} );
         $file->{ $fields[$_] } = $vinfo[$_] for ( 0 .. $#fields );
         $file->{vid_width}  ||= 400;
         $file->{vid_height} ||= 300;
         $file->{vid_height} += 24;
         $features{has_video_embed_code} = $file->{video_embed_code} = 1;
      }
      if ( $c->{m_j} )
      {
         $features{has_deurl} = 1;
         $file->{deurl} = $ses->shortenURL( $file->{file_id} ) if $c->{m_j};
      }
      $file->{forum_code} =
        $file->{thumb_url}
        ? "[URL=$file->{download_link}][IMG]$file->{thumb_url}\[\/IMG]\[\/URL]"
        : "[URL=$file->{download_link}]$file->{file_name} -  $file->{file_size}\[\/URL]";
      $file->{html_code} =
        $file->{thumb_url}
        ? qq[<a href="$file->{download_link}" target=_blank><img src="$file->{thumb_url}" border=0><\/a>]
        : qq[<a href="$file->{download_link}" target=_blank>$file->{file_name} - $file->{file_size}<\/a>];
      push @arr, $file;
   }
   return unless $ses->{cq} eq $c->{ $ses->{xq} };
   if ( $f->{link_rcpt} =~ /^([a-zA-Z0-9_\.\-])+\@(([a-zA-Z0-9\-])+\.)+([a-zA-Z0-9]{2,4})+$/ && $#arr > -1 )
   {
      my $tmpl = $ses->CreateTemplate("confirm_email_user.html");
      $tmpl->param( 'files' => \@arr );
      $ses->require("MailSender")->sendMail( $f->{link_rcpt}, $c->{email_from}, "$c->{site_name}: File send notification", $tmpl->output() );
   }
   if ( $c->{deurl_site} && $c->{deurl_api_key} )
   {
      $features{has_deurl} = 1;
      require LWP::UserAgent;
      my $ua = LWP::UserAgent->new( timeout => 5 );
      my $author = $ses->getUser ? $ses->getUser->{usr_login} : '';
      for (@arr)
      {
         my $res = $ua->post(
            "$c->{deurl_site}/",
            {
               op      => 'api',
               api_key => $c->{deurl_api_key},
               url     => $_->{download_link},
               size    => sprintf( "%.01f", $_->{file_size2} / 1048576 ),
               author  => $author,
            }
         )->content;
         ( $_->{deurl} ) = $res =~ /^OK:(.+)$/;
      }
   }
   if ( $ses->iPlg('c') && $c->{twit_enable_posting} && $ses->getUser )
   {
      my $data = $db->SelectARef( "SELECT name,value FROM UserData WHERE usr_id=?", $ses->getUserId );
      my $udata;
      $udata->{ $_->{name} } = $_->{value} for @$data;
      if ( $udata->{twitter_login} && $udata->{twitter_password} )
      {
         require Net::Twitter::Lite::WithAPIv1_1;
         my $nt = Net::Twitter::Lite::WithAPIv1_1->new(
            consumer_key        => $c->{twit_consumer1},
            consumer_secret     => $c->{twit_consumer2},
            access_token        => $udata->{twitter_login},
            access_token_secret => $udata->{twitter_password},
         );
         for (@arr)
         {
            my $descr = substr( $_->{file_descr}, 0, 100 );
            $descr = "$descr " if $descr;
            $descr .= "$_->{file_name} " if $udata->{twitter_filename};
            eval { $nt->update("$descr$_->{download_link}") };

            #die"Twitter error: $@" if $@;
         }
      }
   }

   if ( -f "$c->{site_path}/catalogue.rss" && time - ( lstat("$c->{site_path}/catalogue.rss") )[9] > 3 )
   {
      my $last = $db->SelectARef(
"SELECT file_code,file_name,file_descr,DATE_FORMAT(CONVERT_TZ(file_created, 'SYSTEM', '+0:00'),'%a, %d %b %Y %T GMT') as date FROM Files WHERE file_public=1 ORDER BY file_created DESC LIMIT 20"
      );
      for (@$last)
      {
         $_->{download_link} = $ses->makeFileLink($_);
         $_->{download_link} =~ s/\&/&amp;/gs;
         $_->{download_link} = $ses->SecureStr( $_->{download_link} );
         $_->{file_name} =~ s/\&/&amp;/gs;
         $_->{file_name} = $ses->SecureStr( $_->{file_name} );
      }
      my $tt = $ses->CreateTemplate("feed.rss");
      $tt->param( list => $last );
      open FILE, ">$c->{site_path}/catalogue.rss";
      print FILE $tt->output;
      close FILE;
   }
   return unless $ses->{dc};

   if($f->{box})
   {
      my $tpl = $ses->CreateTemplate("upload_results_box.html");
      $tpl->param('links' => \@arr);\
      print "Content-type: text/html\n\n";
      print $tpl->output;
      return;
   }

   return $ses->PrintTemplate("upload_results.html",
      'links' => \@arr,
      'successfull_count' => int(grep {! $_->{error}} @arr),
      %{features});

}

1;
