package Engine::Actions::UserPublic;
use strict;

use XFileConfig;
use Engine::Core::Action ( 'IMPLEMENTS' => [qw(load_files_list load_folders_list)] );

sub main
{
   my $user = $db->SelectRow( "SELECT * FROM Users WHERE usr_login=?", $f->{usr_login} );
   return $ses->message($ses->{lang}->{lang_no_such_user}) unless $user;
   $f->{fld} =~ s/\///g;

   my ($folder, $folders, $parent) = _get_folders();

   return $ses->message($ses->{lang}->{lang_no_such_folder}) if $f->{fld} && !$folder;
   my $files = _get_files($user, $folder);

   my $total = $db->SelectOne(
      "SELECT COUNT(*)
                                FROM Files f, Servers s
                                WHERE usr_id=?
                                AND file_public=1
                                AND file_fld_id=?
                                AND f.srv_id=s.srv_id
                                 ORDER BY file_created DESC", $user->{usr_id}, $folder->{fld_id} || 0
   );

   $ses->{page_title} = $ses->{lang}->{lang_files_of} . " " . $user->{usr_login};
   $ses->{page_title} .= ": $folder->{fld_name} folder" if $folder->{fld_name};
   $ses->{meta_descr} = $user->{usr_login} . ' ' . $ses->{lang}->{lang_files};

   $ses->PrintTemplate(
      "user_public.html",
      'login'         => $user->{usr_login},
      'folders'       => $folders,
      'fld_id'        => $f->{fld_id},
      'paging'        => $ses->makePagingLinks( $f, $total ),
      'files_total'   => $total,
      'per_page'      => $c->{items_per_page},
      'folders_total' => $db->SelectOne( "SELECT COUNT(*) FROM Folders WHERE usr_id=?", $ses->getUserId ),
      'token_my_files' => $ses->genToken(op => 'my_files'),
      %$parent,
      files => $files,
   );
}

sub load_folders_list
{
   my ($folder, $folders, $parent) = _get_folders();

   $ses->{form}->{no_hdr} = 1;
   $ses->PrintTemplate(
      'user_public_folders.html',
      %$parent,
      'fld_id'  => $f->{fld_id},
      'folders' => $folders,
   );
   return;
}

sub load_files_list
{
   my $user = $db->SelectRow( "SELECT * FROM Users WHERE usr_login=?", $f->{usr_login} );
   my ($folder, $folders, $parent) = _get_folders();
   my $files = _get_files($user, $folder);

   $ses->{form}->{no_hdr} = 1;
   $ses->PrintTemplate( 'user_public_files.html', files => $files, );
   return;
}

sub _get_folders
{
   my $user = $db->SelectRow( "SELECT * FROM Users WHERE usr_login=?", $f->{usr_login} );
   my $folder = $db->SelectRow( "SELECT * FROM Folders WHERE usr_id=? AND fld_id=?", $user->{usr_id}, $f->{fld_id} );

   my $folders = $db->SelectARef(
      "SELECT *
                                  FROM Folders
                                  WHERE usr_id=?
                                  AND fld_parent_id=?
                                  ORDER BY fld_name" . $ses->makePagingSQLSuffix( $f->{page} ), $user->{usr_id},
      $folder->{fld_id} || 0
   );

   my $parent = $db->SelectRow(
      "SELECT fld_id as fld_parent_id, fld_name as parent_name 
                                FROM Folders 
                                WHERE usr_id=? AND fld_id=?", $user->{usr_id}, $folder->{fld_parent_id}
   ) if $folder->{fld_parent_id};

   for (@$folders)
   {
      $_->{site_url}  = $c->{site_url};
      $_->{usr_login} = $f->{usr_login};
   }

   return ($folder, $folders, $parent);
}

sub _get_files
{
   my ($user, $folder) = @_;
   my $files = $db->SelectARef(
      "SELECT *, TO_DAYS(CURDATE())-TO_DAYS(file_created) as created,
                                       s.srv_htdocs_url
                                FROM Files f, Servers s
                                WHERE usr_id=?
                                AND file_public=1
                                AND file_fld_id=?
                                AND f.srv_id=s.srv_id
            ORDER BY file_created DESC" . $ses->makePagingSQLSuffix( $f->{page} ), $user->{usr_id}, $folder->{fld_id} || 0
   );

   my $cx = 0;

   for (@$files)
   {
      $_->{site_url} = $c->{site_url};

      my $file_name = $_->{file_name};
      utf8::decode($file_name);
      $_->{file_name_txt} =
        length($file_name) > $c->{display_max_filename}
        ? substr( $file_name, 0, $c->{display_max_filename} ) . '&#133;'
        : $file_name;
      utf8::encode( $_->{file_name_txt} );

      $_->{file_size}     = $ses->makeFileSize( $_->{file_size} );
      $_->{download_link} = $ses->makeFileLink($_);
      my ($ext) = $_->{file_name} =~ /\.(\w+)$/i;
      $ext = lc $ext;

      $_->{img_preview} =
        $ext =~
/^(ai|aiff|asf|avi|bmpbz2|css|doc|eps|gif|gz|html|jpg|jpeg|mid|mov|mp3|mpg|mpeg|ogg|pdf|png|ppt|ps|psd|qt|ra|ram|rm|rpm|rtf|tgz|tif|torrent|txt|wav|xls|xml|zip|7z|exe|flv|swf|qma|wmv|mkv|rar)$/
        ? "$c->{site_url}/images/icons/$ext-dist.png"
        : "$c->{site_url}/images/icons/default-dist.png";
      $_->{add_to_account} = 1 if $ses->getUser && $_->{usr_id} != $ses->getUserId;
      if (  ( $c->{m_i} && $_->{file_name} =~ /\.(jpg|jpeg|gif|png|bmp)$/i )
         || ( $c->{m_v} && $_->{file_name} =~ /\.(avi|divx|flv|mp4|wmv|mkv)$/i ) )
      {
         my $iurl = $_->{srv_htdocs_url};
         $iurl =~ s/^(.+)\/.+$/$1\/i/;
         my $dx = sprintf( "%05d", ( $_->{file_real_id} || $_->{file_id} ) / $c->{files_per_folder} );
         $_->{img_preview2} = "$iurl/$dx/$_->{file_real}_t.jpg";
      }
      $_->{'tr'} = 1 if ++$cx % 3 == 0;
   }

   return $files;
}

1;
