package Engine::Components::AffTracker;
use strict;
use vars qw($ses $db $c $f);
use HCE_MD5;
use Digest::MD5 qw(md5 md5_hex);
use MIME::Base64 qw(encode_base64 decode_base64);
use CheckParams;

use lib "Modules";
use JSON;

sub get {
   my ($self) = @_;
   my $stamp = $ses->getCookie('affiliate') || return;
   return $self->decode($stamp);
}

sub set {
   my ($self, $opts) = @_;
   check_params($opts, qw(aff_id file_id referer));

   my $new = { %$opts };
   $new->{time} = time();

   my $payload = JSON->new->canonical->encode($new);
   my $sign = $self->sign($payload);
   my $hce = HCE_MD5->new($c->{dl_key},"XFileSharingPRO");
   my $hash = encode_base64( $hce->hce_block_encrypt($sign . $payload), "");

   $ses->setCookie("affiliate", $hash, "+14d");
}

sub decode {
   my ($self, $stamp) = @_;

   my $hce = HCE_MD5->new($c->{dl_key},"XFileSharingPRO");
   my $s = $hce->hce_block_decrypt(decode_base64($stamp));
   my $sign = substr($s, 0, 8);
   my $payload = substr($s, 8);
   my $computed_sign = $self->sign($payload);
   return if $computed_sign ne $sign;

   return eval { JSON::decode_json($payload) };
}

sub sign {
   my ($self, $payload) = @_;
   return substr(md5($payload . $c->{dl_key}), 0, 8);
}

1;
