package Engine::Components::DlTracker;
use strict;
use vars qw($ses $db $c $f);
use CheckParams;

sub downloadChecks {
   my ($self, $file) = @_;

   my $owner = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $file->{usr_id});
   my $max_dl_size = $self->getMaxDLSize($owner);

   if($max_dl_size && $file->{file_size} > $max_dl_size*1048576)
   {
      $file->{message} = "You can download files up to $max_dl_size Mb only.<br>Upgrade your account to download bigger files.";
   }

   if($c->{max_downloads_number} && $file->{file_downloads} >= $c->{max_downloads_number})
   {
      $file->{message} = "This file reached max downloads limit";
   }

   if($c->{file_dl_delay})
   {
      my $cond = $ses->getUser ? "usr_id=".$ses->getUserId : "ip='".$ses->getIP."'";
      my $last = $db->SelectRow("SELECT *, UNIX_TIMESTAMP()-UNIX_TIMESTAMP(created) as dt 
                                 FROM IP2Files WHERE $cond 
                                 ORDER BY created DESC LIMIT 1");
      my $wait = $c->{file_dl_delay} - $last->{dt};
      if($last->{dt} && $wait>0)
      {
         require Time::Elapsed;
         my $et = new Time::Elapsed;
         my $elapsed = $et->convert($wait);
         $file->{message}  = "You have to wait $elapsed till next download";
         $file->{message} .= "<br><br>Download files instantly with <a href='$c->{site_url}/premium/'>Premium-account</a>" if $c->{enabled_prem};
      }
   }

   if($c->{add_download_delay})
   {
      my $cond = $ses->getUser ? "usr_id=".$ses->getUserId : "ip='".$ses->getIP."'";
      my $last = $db->SelectRow("SELECT *, UNIX_TIMESTAMP()-UNIX_TIMESTAMP(created) as dt 
                                 FROM IP2Files WHERE $cond 
                                 ORDER BY created DESC LIMIT 1");
      my $wait = int($c->{add_download_delay}*$last->{size}/(100*1048576)) - $last->{dt};
      if($wait>0)
      {
         require Time::Elapsed;
         my $et = new Time::Elapsed;
         my $elapsed = $et->convert($wait);
         $file->{message}  = "You have to wait $elapsed till next download";
         $file->{message} .= "<br><br>Download files instantly with <a href='$c->{site_url}/premium/'>Premium-account</a>" if $c->{enabled_prem};
      }
   }

   if($ses->getUserLimit('bw_limit'))
   {
      $file->{message} = "You have reached the download-limit: $c->{bw_limit} Mb for last $c->{bw_limit_days} days"
         if $ses->require("TrafficRegistry")->getUserBandwidth($c->{bw_limit_days}) > $ses->getUserLimit('bw_limit');
   }

   if(($owner->{usr_premium_only} || $file->{file_premium_only}) && $ses->{utype} ne 'prem')
   {
      $ses->PrintTemplate("download_premium_only.html",%$file), return;
   }

   return $file;
}

sub downloadTrack {
   my ($self, $file) = @_;
   my $usr_id = $ses->getUser ? $ses->getUserId : 0;
   
   if(!$db->SelectOne("SELECT file_id FROM IP2Files WHERE file_id=? AND ip=? AND usr_id=?",$file->{file_id},$ses->getIP,$usr_id))
   {
      $f->{referer}||= $ses->getCookie('ref_url') || $ses->getEnv('HTTP_REFERER');
      $f->{referer}=~s/$c->{site_url}//i;
      $f->{referer}=~s/^https?:\/\///i;

      if($c->{m_n_100_complete_percent})
      {
         # Don't charge any money, leave it for fs.cgi instead
         $db->Exec("INSERT INTO IP2Files SET 
               file_id=?, usr_id=?, owner_id=?, ip=?, size=0, referer=?, status=?",      
               $file->{file_id},$usr_id||0,$file->{usr_id}||0,$ses->getIP,$f->{referer}||'',
               ($f->{adblock_detected} ? 'ADBLOCK' : 'Not completed'));
         return;
      }

      $self->countDL($file,
         adblock_detected => $f->{adblock_detected}||'',
         referer => $f->{referer},
         ip => $ses->getIP,
         usr_id => $ses->getUser ? $ses->getUserId : 0);
   }
}

sub isVipFile
{
   my ($self, $file) = @_;
   return 0 if $file->{file_price} <= 0;

   my $owner = $Engine::Core::db->SelectRow("SELECT *, usr_premium_expire > NOW() AS is_premium FROM Users WHERE usr_id=?", $file->{usr_id});
   return 1 if $owner->{usr_allow_vip_files};

   my $owner_utype = $owner->{is_premium} ? 'prem' : 'reg';
   return $c->{"allow_vip_files_$owner_utype"};
}

sub countDL {
   my ($self, $file, %opts) = @_;
   check_params(\%opts, qw(adblock_detected referer ip usr_id));

   my $profits_charger = $ses->require("ProfitsCharger");
   my ($status, $money) = $profits_charger->calcDlTier($file, %opts);
   my $money = sprintf("%.05f",$money);

   # Charge webmaster profits
   my $website = $db->SelectRow("SELECT * FROM Websites WHERE domain=?", $ses->getDomain($opts{referer})) if $money > 0 && $opts{referer};
   $profits_charger->payToUser($ses->getUserId() => $website->{usr_id}, $money * $c->{m_x_rate} / 100, 'site') if $website;

   # Charge referal profits
   my $aff_id = $db->SelectOne("SELECT usr_aff_id FROM Users WHERE usr_id=?",$file->{usr_id}) if $money > 0 && $file->{usr_id};
   $profits_charger->payToUser($file->{usr_id} => $aff_id, $money * $c->{referral_aff_percent} / 100, 'refs') if $aff_id;

   $db->Exec("INSERT INTO IP2Files 
              SET file_id=?, 
                  usr_id=?, 
                  owner_id=?, 
                  ip=?, 
                  size=?, 
                  money=?,
                  referer=?,
                  status=?,
                  finished=1
               ON DUPLICATE KEY UPDATE
                  money=?,
                  status=?,
                  finished=1",
               $file->{file_id},
               $opts{usr_id},
               $file->{usr_id}||0,
               $opts{ip},
               $file->{file_size},
               $money,
               $opts{referer}||'',
               $status,
               $money,
               $status);

   $db->Exec("UPDATE LOW_PRIORITY Files 
              SET file_downloads=file_downloads+1, 
                  file_money=file_money+?, 
                  file_last_download=NOW() 
              WHERE file_id=?",$money,$file->{file_id});

   if($c->{hold_profits_interval})
   {
      $db->Exec("INSERT INTO HoldProfits SET day=CURDATE(), usr_id=?, amount=?
         ON DUPLICATE KEY UPDATE amount=amount+?",
         $file->{usr_id}, $money, $money) if $file && $file->{usr_id};
   }
   else
   {
      $db->Exec("UPDATE LOW_PRIORITY Users SET usr_money=usr_money+? WHERE usr_id=?",$money,$file->{usr_id}) if $file->{usr_id} && $money;
      $db->Exec(sprintf("INSERT INTO Stats SET day=CURDATE(), paid_to_users='%0.8f' ON DUPLICATE KEY UPDATE paid_to_users=paid_to_users+'%0.8f'", ($money) x 2));
   }

   if($c->{m_p_show_downloads_mode} ne 'show_only_paid' || $money > 0)
   {
      $db->Exec("INSERT INTO Stats2
                 SET usr_id=?, day=CURDATE(),
                     downloads=1, profit_dl=$money
                 ON DUPLICATE KEY UPDATE
                     downloads=downloads+1, profit_dl=profit_dl+$money
                ",$file->{usr_id}) if $file->{usr_id};
   }

   $db->Exec("INSERT INTO Stats SET day=CURDATE(), downloads=1,bandwidth=? ON DUPLICATE KEY UPDATE downloads=downloads+1,bandwidth=bandwidth+?",
      $file->{file_size}, $file->{file_size});
}

sub getMaxDLSize
{
   my ($self, $owner) = @_;
   my $usr_aff_max_dl_size = $owner->{usr_aff_max_dl_size} if $owner && $owner->{usr_aff_enabled} && $Engine::Core::ses->{utype} ne 'prem';
   return $usr_aff_max_dl_size || $c->{max_download_filesize};
}

1;
