package Engine::Components::MailSender;
use strict;
use vars qw($ses $db $c $f);

sub sendMail
{
   my $self = shift;
   my ($mail_to, $mail_from, $subject, $body) = @_;

   if($c->{smtp_auth} eq 'tls')
   {
      return $self->sendMailTLS(@_);
   }

   my $content_type = $c->{email_text} ? 'text/plain' : 'text/html';
   require MIME::Lite;
   my $msg = MIME::Lite->new(
        From    => $mail_from,
        To      => $mail_to,
        Subject => $subject,
        Data    => $body
   );
   $msg->attr("content-type"         => $content_type);
   $msg->attr("content-type.charset" => "UTF-8");

   if($c->{smtp_server} && $c->{smtp_user} && $c->{smtp_pass})
   {
      $msg->send('smtp',$c->{smtp_server}, AuthUser=>$c->{smtp_user}, AuthPass=>$c->{smtp_pass} );
   }
   elsif($c->{smtp_server})
   {
      $msg->send('smtp',$c->{smtp_server});
   }
   else
   {
      $msg->send();
   }
}

sub sendMailTLS
{
   require Net::SMTPS;
   require Authen::SASL;

   my ($self, $mail_to, $mail_from, $subject, $body) = @_;
   my $content_type = $c->{email_text} ? 'text/plain' : 'text/html';

   my $msg = new Net::SMTPS(
                   $c->{smtp_server},
                   Timeout =>	30,
                   Port => $c->{smtp_port}||587,
                   doSSL => 'starttls',
                   Debug => $c->{smtp_debug});

   ## Requires Authen::SASL
   $msg->auth($c->{smtp_user}, $c->{smtp_pass}) || die("could not authenticate\n") if $c->{smtp_user};

   $msg->mail($mail_from);
   $msg->recipient($mail_to);

   $msg->data();
   $msg->datasend("To: $mail_to\n");
   $msg->datasend("From: $mail_from\n");
   $msg->datasend("Content-Type: $content_type; charset=UTF-8\n");
   $msg->datasend("Subject: $subject\n\n");
   $msg->datasend($body);
   $msg->dataend();

   $msg->quit;
}

1;
