package Engine::Components::PremiumComparison;
use strict;
use vars qw($ses $db $c $f);

sub mkPremiumComparison
{
   # Do not modify $c directly to prevent affecting FastCGI

   my $limits = {%$c};
   for my $x ('max_upload_filesize')
   {
      for my $y ( 'anon', 'reg', 'prem' )
      {
         my $z = "$x\_$y";
         $limits->{$z} = $c->{$z} ? "$c->{$z} Mb" : "Unlimited";
      }
   }
   $limits->{max_downloads_number_reg}  = $c->{max_downloads_number_reg}  || 'Unlimited';
   $limits->{max_downloads_number_prem} = $c->{max_downloads_number_prem} || 'Unlimited';
   $limits->{files_expire_anon} =
     $c->{files_expire_access_anon}
     ? "$c->{files_expire_access_anon} $ses->{lang}->{lang_days_after_downl}"
     : $ses->{lang}->{lang_never};
   $limits->{files_expire_reg} =
     $c->{files_expire_access_reg}
     ? "$c->{files_expire_access_reg} $ses->{lang}->{lang_days_after_downl}"
     : $ses->{lang}->{lang_never};
   $limits->{files_expire_prem} =
     $c->{files_expire_access_prem}
     ? "$c->{files_expire_access_prem} $ses->{lang}->{lang_days_after_downl}"
     : $ses->{lang}->{lang_never};

   $limits->{disk_space_reg}  = $c->{disk_space_reg}  ? sprintf( "%.0f GB", $c->{disk_space_reg} / 1024 )  : "Unlimited";
   $limits->{disk_space_prem} = $c->{disk_space_prem} ? sprintf( "%.0f GB", $c->{disk_space_prem} / 1024 ) : "Unlimited";

   $limits->{bw_limit_anon} =
     $c->{bw_limit_anon}
     ? sprintf( "%.0f GB", $c->{bw_limit_anon} / 1024 ) . " in $c->{bw_limit_days} $ses->{lang}->{lang_days}"
     : 'Unlimited';
   $limits->{bw_limit_reg} =
     $c->{bw_limit_reg}
     ? sprintf( "%.0f GB", $c->{bw_limit_reg} / 1024 ) . " in $c->{bw_limit_days} $ses->{lang}->{lang_days}"
     : 'Unlimited';
   $limits->{bw_limit_prem} =
     $c->{bw_limit_prem}
     ? sprintf( "%.0f GB", $c->{bw_limit_prem} / 1024 ) . " in $c->{bw_limit_days} $ses->{lang}->{lang_days}"
     : 'Unlimited';

   for my $utype (qw(anon reg prem))
   {
      $limits->{"download_resume_$utype"} = $c->{m_n} ? $c->{"m_n_dl_resume_$utype"} : $c->{"direct_links_$utype"};
   }

   return $limits;
}

1;
