package Engine::Components::ProfitsCharger;
use strict;
use vars qw($ses $db $c $f);

sub calcDlTier {
   my ($self, $file, %opts) = @_;

   my $money = 0;

   if($ses->iPlg('p') && -e "$c->{cgi_path}/GeoLite2-Country.mmdb")
   {
      my $size_id;
      my @ss = split(/\|/,$c->{tier_sizes});
      for(0..5){$size_id=$_ if defined $ss[$_] && $file->{file_size}>=$ss[$_]*1024*1024;}
      require Geo::IP2;
      my $gi = Geo::IP2->new("$c->{cgi_path}/GeoLite2-Country.mmdb");
      my $country = $gi->country_code_by_addr($opts{ip});
      if(defined $size_id)
      {
        my @tier_ids = sort { $a <=> $b } map { /^tier(\d+)_money/ } keys(%$c);
        my @tiers = map { { countries => $c->{"tier$_\_countries"}, money => $c->{"tier$_\_money"} } } @tier_ids;
        my ($tier) = grep { length($_->{countries}) && $country =~ /^$_->{countries}$/ } @tiers;
        $tier ||= $tiers[-1];

        $money = (split(/\|/,$tier->{money}))[$size_id];
        return('REWARD_NOT_SPECIFIED') if !$money;
      }

      return('MAX24_REACHED') if $c->{max_money_last24} && $db->SelectOne("SELECT SUM(money) FROM IP2Files WHERE ip=? AND created>NOW()-INTERVAL 24 HOUR",$opts{ip}) >= $c->{max_money_last24};
      return('MAX24_REACHED') if $c->{max_paid_dls_last24} && $db->SelectOne("SELECT COUNT(*) FROM IP2Files WHERE ip=? AND created>NOW()-INTERVAL 24 HOUR AND money > 0",$opts{ip}) >= $c->{max_paid_dls_last24};
   }
   else
   {
      $money = $ses->getUser && $ses->getUser->{premium} ? $c->{dl_money_prem} : $c->{dl_money_reg};
      $money = $c->{dl_money_anon} unless $ses->getUser;
      return('REWARD_NOT_SPECIFIED') if !$money;
      return('FILE_TOO_SMALL') if $file->{file_size} < $c->{money_filesize_limit}*1024*1024;
   }
   $money = $money / 1000;

   return("OWN_FILE") if $opts{usr_id} && $file->{usr_id}==$opts{usr_id} && $c->{no_money_from_uploader_user};
   return("OWN_IP") if $file->{file_ip} eq $opts{ip} && $c->{no_money_from_uploader_ip};
   return('ADBLOCK') if $opts{adblock_detected};
   return('GEOIP_MISSING') if $ses->iPlg('p') && ! -e "$c->{cgi_path}/GeoLite2-Country.mmdb";

   my $owner = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $file->{usr_id}) if $file->{usr_id};
   if($owner && $ses->iPlg('p'))
   {
      my $profit_mode = lc($owner->{usr_profit_mode});
      my $perc = $c->{"m_y_$profit_mode\_dl"};
      return('NO_MIXED_PERCENT') if !$perc && $profit_mode eq 'mix';
      return('NOT_PPD') if !$perc && $profit_mode ne 'ppd';
      return("NO_PERCENT_SPECIFIED: ".$profit_mode) if !$perc;

      if($c->{m_y_manual_approve} && !$owner->{usr_aff_enabled})
      {
         return("NOT_APPROVED_AFFILIATE");
         $perc = 0;
      }

      $money = $money*$perc/100;
      return("NOT_PPD") if !$money && $profit_mode ne 'ppd';
   }
   
   return ('Completed', sprintf("%.05f",$money));
}

sub chargeAffs
{
   # Charges all affs that's expecting a reward for $transaction
   my ($self, $transaction) = @_;

   # Is that a sale or a rebill?
   my $prev_transaction = $db->SelectRow("SELECT *, TIMESTAMPDIFF(DAY, created, NOW()) AS elapsed
         FROM Transactions
         WHERE usr_id=?
         AND id!=?
         AND verified
         ORDER BY created DESC",
         $transaction->{usr_id},
         $transaction->{id});

   my $is_rebill = $prev_transaction && $prev_transaction->{elapsed} <= 31 ? 1 : 0;
   my $transaction_age = $db->SelectOne("SELECT TIMESTAMPDIFF(DAY, created, NOW())
      FROM Transactions
      WHERE id=?",
      $transaction->{id});

   $db->Exec("UPDATE Transactions SET rebill=?, domain=? WHERE id=?",
      $is_rebill, $ses->getDomain($transaction->{ref_url}), $transaction->{id});

   $is_rebill = 1 if $transaction_age > 3; # Must be exactly here in order to keep initial transaction in 'sales' section of detailed stats
   my $stats = $is_rebill ? 'rebills' : 'sales';

   my $payment_settings = $db->SelectRow("SELECT * FROM PaymentSettings WHERE name=?", $transaction->{plugin});
   my $netto_amount = ($payment_settings && $payment_settings->{commission})
      ? $transaction->{amount} * (1 - $payment_settings->{commission} / 100)
      : $transaction->{amount};

   my $uploader = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $transaction->{aff_id}) if $transaction->{aff_id};
   my $uploader_percent = $uploader->{"usr_$stats\_percent"} || $c->{"m_y_".lc($uploader->{usr_profit_mode})."_$stats"} || $c->{sale_aff_percent} if $uploader;
   my $uploader_profits = $netto_amount * $uploader_percent / 100 if $uploader && (!$c->{m_y_manual_approve} || $uploader->{usr_aff_enabled});
   print STDERR "Profit mode=$uploader->{usr_profit_mode}, stats = $stats, aff percent = $uploader_percent\n" if $uploader;

   my $webmaster = $db->SelectRow("SELECT * FROM Users WHERE usr_id=(SELECT usr_id FROM Websites WHERE domain=?)", $ses->getDomain($transaction->{ref_url}));
   my $webmaster_profits = $netto_amount * ($webmaster->{usr_m_x_percent} || $c->{m_x_rate}) / 100 if $webmaster;

   my %info = ( transaction_id => $transaction->{id} );
   $self->payToUser($transaction->{usr_id} => $transaction->{aff_id}, $uploader_profits, $stats, %info);
   $self->payToUser($transaction->{usr_id} => $webmaster->{usr_id}, $webmaster_profits, 'site', %info);

   my $ref1 = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $uploader->{usr_aff_id}) if $uploader;
   my $ref2 = $db->SelectRow("SELECT * FROM Users WHERE usr_id=?", $webmaster->{usr_aff_id}) if $webmaster;

   $self->payToUser($transaction->{aff_id} => $ref1->{usr_id}, $uploader_profits * $c->{referral_aff_percent} / 100, 'refs', %info) if $ref1;
   $self->payToUser($transaction->{aff_id} => $ref2->{usr_id}, $webmaster_profits * $c->{referral_aff_percent} / 100, 'refs', %info) if $ref2;
}

sub payToUser {
   my ($self, $generated_by, $usr_id_to, $amount, $reason, %opts) = @_;
   return if !$usr_id_to || $amount <= 0;

   print STDERR "Charging usr_id=$usr_id_to with \$$amount (stats = $reason)\n";
   
   if($c->{hold_profits_interval})
   {
      $db->Exec("INSERT INTO HoldProfits SET day=CURDATE(), usr_id=?, amount=?
         ON DUPLICATE KEY UPDATE amount=amount+?",
         $usr_id_to, $amount, $amount);
   }
   else
   {
      $db->Exec("UPDATE Users SET usr_money=usr_money+? WHERE usr_id=?", $amount, $usr_id_to);
   }
   
   $db->Exec("INSERT INTO PaymentsLog SET usr_id_from=?, usr_id_to=?, type=?, amount=?, transaction_id=?, created=NOW()",
      $generated_by||0, $usr_id_to, $reason, $amount, $opts{transaction_id}||'');
   
   my $statsTracker = $ses->require("StatsTracker");
   $statsTracker->registerEvent('profits_received', { amount => $amount, stats => $reason, usr_id => $usr_id_to });
}

1;
