package Engine::Components::ServersRegistry;
use strict;
use vars qw($ses $db $c $f);
use CheckParams;

sub createServer
{
   my ($class, $opts) = @_;

   my @required = qw(srv_name srv_ip srv_cgi_url srv_htdocs_url srv_disk_max srv_key);
   my @defined = qw(srv_status srv_allow_regular srv_allow_premium srv_torrent srv_countries srv_cdn srv_ftp);
   check_params($opts, map { "*$_" } @required, @defined);

   $opts->{srv_status}||='ON';
   $opts->{srv_allow_premium} = 1 if !defined($opts->{srv_allow_premium});
   $opts->{srv_allow_regular} = 1 if !defined($opts->{srv_allow_regular});

   my @sflds = (@required, @defined);
   $db->Exec( "INSERT INTO Servers SET srv_created=CURDATE(), " . join( ',', map { "$_=?" } @sflds ), map { $opts->{$_} || '' } @sflds );
   return $db->getLastInsertId;
}

sub selectServer
{
   my ($class, $user, @filters) = @_;

   my @custom_filters = map { " AND $_"} @filters;
   my $type_filter = $user && $user->{exp_sec} > 0 ? "AND srv_allow_premium=1" : "AND srv_allow_regular=1";
   my $country_filter;

   if($c->{m_g} && -f "$c->{cgi_path}/GeoLite2-Country.mmdb")
   {
      require Geo::IP2;
      my $gi = Geo::IP2->new("$c->{cgi_path}/GeoLite2-Country.mmdb");
      my $country = $gi->country_code_by_addr($ses->getIP);
      $country_filter="AND srv_countries LIKE '%$country%'" if $country;
      $country_filter||="AND srv_countries=''";
   }

   my $server = $ses->db->SelectRow("SELECT * FROM Servers 
                                WHERE srv_status='ON' 
                                AND srv_disk+? <= srv_disk_max*0.99
                                $type_filter
                                $country_filter
                                @custom_filters
                                ORDER BY RAND()
                                LIMIT 1",$c->{max_upload_filesize}||100);
   if($c->{m_g} && !$server)
   {
      $server = $ses->db->SelectRow("SELECT * FROM Servers 
                                WHERE srv_status='ON' 
                                AND srv_disk+? <= srv_disk_max*0.98
                                $type_filter
                                @custom_filters
                                ORDER BY RAND()
                                LIMIT 1",$c->{max_upload_filesize}||100);
   }

   return $server;
}

1;
