package Engine::Components::UpdatesChecker;
use strict;
use vars qw($ses $db $c $f);
use LWP::UserAgent;
use JSON;
use Installer;

sub checkUpdates {
   my $res = '[
   {
      "enabled" : 0,
      "letter" : "r",
      "package_docs" : "https://sibsoft.net/xfilesharing/mods/url_leech.html",
      "package_id" : "mod_leech",
      "package_name" : "URL Leech Mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "3.3"
   },
   {
      "enabled" : 0,
      "letter" : "b",
      "package_docs" : "http://sibsoft.net/xfilesharing/mods/archives.html",
      "package_id" : "mod_archives",
      "package_name" : "Archives mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "3.3"
   },
   {
      "enabled" : 1,
      "letter" : "4",
      "package_docs" : "https://sibsoft.net/xfilesharing/mods/cloud.html",
      "package_id" : "mod_webdav",
      "package_name" : "Cloud Drive mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "3.3"
   },
   {
      "enabled" : 0,
      "letter" : "t",
      "package_docs" : "https://sibsoft.net/xfilesharing/mods/torrents.html",
      "package_id" : "mod_torrent",
      "package_name" : "Torrents Mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "4.6"
   },
   {
      "enabled" : 1,
      "letter" : "v",
      "package_docs" : "https://sibsoft.net/xfilesharing/mods/video.html",
      "package_id" : "mod_videncode",
      "package_name" : "Video & MP4 Encoding Mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "3.3"
   },
   {
      "enabled" : 0,
      "letter" : "8",
      "package_docs" : "https://sibsoft.net/xfilesharing/mods/backup.html",
      "package_id" : "mod_backup",
      "package_name" : "Backup Mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "3.3"
   },
   {
      "enabled" : 1,
      "letter" : "m",
      "package_docs" : "https://sibsoft.net/xfilesharing/mods/payments.html",
      "package_id" : "mod_payments",
      "package_name" : "Payment Gateways mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "3.3"
   },
   {
      "enabled" : 1,
      "letter" : "7",
      "package_docs" : "http://sibsoft.net/xfilesharing/mods/api.html",
      "package_id" : "mod_api",
      "package_name" : "API Mod",
      "package_vendor" : "Sibsoft Ltd",
      "package_version" : "3.3"
   }
]';
   my @packages = @{ JSON::decode_json(Installer::remove_newlines($res)) };

   my $servers = $db->SelectARef("SELECT srv_id, srv_name FROM Servers WHERE srv_status != 'OFF'");
   unshift @$servers, { srv_id => 0, srv_name => 'Main' };

   for my $pkg (@packages) {
      my %installed_versions;
      for(@$servers) {
         my $version = $db->SelectOne("SELECT installed_version FROM PkgVersions WHERE package_id=? AND srv_id=?",
            $pkg->{package_id}, $_->{srv_id});
         push @{ $installed_versions{$version || '0.0'} }, $_;
      }

      $pkg->{installed_versions} = [ map { { installed_version => $_, servers => $installed_versions{$_} } } keys(%installed_versions) ];
      $pkg->{todo} = [];

      for(@{ $pkg->{installed_versions} }) {
         push @{ $pkg->{todo} }, @{ $_->{servers} } if $pkg->{enabled} && cmp_versions($_->{installed_version}, $pkg->{package_version}) == -1;
      }

      $pkg->{up_to_date} = $pkg->{enabled} && @{ $pkg->{todo} } == 0;
      $pkg->{to_update} =  $pkg->{enabled} && @{ $pkg->{todo} } > 0;

      my @sorted = sort { cmp_versions($a, $b) } map { $_->{installed_version} } @{ $pkg->{installed_versions} };
      my $min = $sorted[0];
      my $max = $sorted[$#sorted];
      $pkg->{installed_version} = $min eq $max ? $max : "$min - $max";
      $pkg->{installed_version} = '' if $pkg->{installed_version} eq '0.0';
   }

   my $packages_to_update = grep { $_->{to_update} } @packages;
   $db->Exec("INSERT INTO Misc SET name='packages_to_update', value=? ON DUPLICATE KEY UPDATE value=?", ($packages_to_update) x 2);
   
   return @packages;
}

sub cmp_versions {
   my @a = split(/\./, shift());
   my @b = split(/\./, shift());

   push @a, 0 while @a < @b;
   push @b, 0 while @b < @a;

   while(@a && @b) {
      my $ret = pop(@a) cmp pop(@b);
      return $ret if $ret;
   }

   return 0;
}

1;
