package Engine::Components::VideoMakeCode;
use strict;
use vars qw($ses $db $c $f);
use List::Util qw(max);

sub makeCode {
   my ($self, $file, $gen) = @_;

   my ($ext) = $file->{file_name}=~/\.(\w+)$/i;

   return $file if $file->{file_name} !~ /\.(avi|divx|xvid|mpg|mpeg|vob|mov|3gp|flv|mp4|wmv|mkv)$/i;

   $file->{no_link}=1 if $c->{video_mod_no_download};

   # Parsing file_spec
   my @fields=qw(vid vid_length vid_width vid_height vid_bitrate vid_audio_bitrate vid_audio_rate vid_codec vid_audio_codec vid_fps);
   my @vinfo = split(/\|/,$file->{file_spec});
   $file->{$fields[$_]}=$vinfo[$_] for (0..$#fields);

   # Thumbs
   my $dx = sprintf("%05d",($file->{file_real_id}||$file->{file_id})/$c->{files_per_folder});
   $file->{srv_htdocs_url}=~/(.+)\/.+$/;
   $file->{video_img_url}="$1/i/$dx/$file->{file_real}.jpg";
   $file->{video_thumb_url}="$1/i/$dx/$file->{file_real}_t.jpg";

   # Dimensions
   ($file->{vid_width},$file->{vid_height})=($c->{m_v_width},$c->{m_v_height}) if $c->{m_v_width} && $c->{m_v_height};
   $file->{vid_width}=$f->{w} if $f->{w};
   $file->{vid_height}=$f->{h} if $f->{h};
   $file->{vid_height2} = $file->{vid_height}+20;
   $file->{vid_length2} = sprintf("%02d:%02d:%02d",int($file->{vid_length}/3600),int(($file->{vid_length}%3600)/60),$file->{vid_length}%60);

   return $file unless $gen;

   # Direct link
   $ext = 'mp4' if $file->{file_size_encoded};
   my $direct_link = $Engine::Core::ses->getPlugins('CDN')->genDirectLink($file,
      encoded => 1,
      file_name => "video.$ext",
      accept_ranges => 1,
      limit_conn => max( $c->{m_n_limit_conn}, 10 ));
   return if !$direct_link;
   $file->{video_code} = $Engine::Core::ses->getPlugins('Video')->makeCode($file, $direct_link);

   # Ads overlay mod
   if($c->{m_a} && $file->{video_code})
   {
      $file->{m_a_css}="document.write('<Style>#player_img {position:absolute;}
a#vid_play {background: repeat scroll center top; display:block; position:absolute; top:50%; margin-top:-30px; left:15%; margin-left:-30px; z-index: 99; width: 60px; height: 60px;}
a#vid_play:hover {background-position:bottom;}
#player_ads {position:absolute; top:0px; left:30%; width:70%; height:100%; z-index:2;}
#player_code {visibility: hidden;}</Style>');";
      $file->{m_a_css} = $Engine::Core::ses->encodeJS($file->{m_a_css});
   }

   return $file;
}

1;
