package Engine::Cronjobs::FetchDMCAReports;
use strict;

use XFileConfig;
use Engine::Core::Cronjob;
use IO::Socket::SSL;
use MIME::Base64 qw(decode_base64);

sub main
{
   return if !$c->{m_d} || !$c->{dmca_mail_host};
   print "Fetching DMCA reports...\n";

   my $client = getIMAPClient();

   my @unseen = $client->search('UNSEEN');
   my $domain = $1 if $c->{site_url} =~ /^https?:\/\/([^\/]+)/;

   for my $msg_id (@unseen)
   {
      $client = getIMAPClient() if !$client->IsConnected();

      print "Fetching #$msg_id from $c->{dmca_mail_host}...\n";

      my $body = $client->body_string($msg_id);
      my $from_base64 = decode_base64($body);
      $body .= "\nDecoded from base64:\n$from_base64" if length($from_base64) > 16;

      my $headers = $client->parse_headers( $msg_id, 'From', 'Subject' );
      my $from    = $headers->{'From'}->[0];
      my $subject = $ses->SecureStr( $headers->{'Subject'}->[0] );

      $client->set_flag( "Seen", $msg_id );

      while ( $body =~ /\/(\w{12})/g ) {
         addReport($from, $subject, $body, $db->SelectRow( "SELECT * FROM Files WHERE file_code=?", $1 ));
      }

      if($c->{m_j}) {
         require Math::Base62;

         while ( $body =~ /\/d\/(\w{1,16})/g ) {
            my $file_id = Math::Base62::decode_base62( $1 );
            addReport($from, $subject, $body, $db->SelectRow( "SELECT * FROM Files WHERE file_id=?", $file_id ));
         }
      }

      if($c->{m_n}) {
         require HCE_MD5;

         while ( $body =~ /\/d\/(\w{16,})/g ) {
            my $hce = HCE_MD5->new($c->{dl_key}, "XFileSharingPRO");
            my ($srv_id, $file_id) = unpack("SL", $hce->hce_block_decrypt($ses->decode32($1)) );
            addReport($from, $subject, $body, $db->SelectRow( "SELECT * FROM Files WHERE file_id=?", $file_id ));
         }
      }
   }
}

sub addReport {
   my ($from, $subject, $body, $file) = @_;
   return if !$file;
   return if $db->SelectOne( "SELECT * FROM Reports WHERE file_id=? AND status='PENDING'", $file->{file_id} );    # Already reported

   print "Placing file in reported list: ", $ses->makeFileLink($file), "<br>\n";

   my ( $name, $email ) = map { $ses->SecureStr($_) } $from =~ /(.*) <(.*)>/;

   $db->Exec(
      "INSERT INTO Reports SET file_id=?, usr_id=?, filename=?, name=?, email=?, reason=?, info=?, ip=?, status='PENDING', created=NOW()",
      $file->{file_id},
      $file->{usr_id},
      $file->{file_name},
      $name||'',
      $email||'',
      "$subject (from $c->{dmca_mail_host})",
      $ses->SecureStr($body),
      '0.0.0.0'
   );
}

sub getIMAPClient {
   print "Connecting...\n";

   require Mail::IMAPClient;

   my $client = Mail::IMAPClient->new(
      Server   => $c->{dmca_mail_host},
      User     => $c->{dmca_mail_login},
      Password => $c->{dmca_mail_password},
      Port     => 993,
      Ssl      => 1,
      Uid      => 1,
      Keepalive => 1,
      Socketargs => [ "SSL_verify_mode" => SSL_VERIFY_NONE ],
   );

   $client->connect() || die("Could not connect: $@");
   $client->select('INBOX');
}

1;
