package Engine::FSRPC::RarProcess;
use strict;

use XFSConfig;
use SafeRun;
use XUpload;
use File::Temp;
use MultiHDD;

sub main
{
   my ($f) = @_;
   my $dx = sprintf("%05d",$f->{file_id}/$c->{files_per_folder});
   my $src_disk_id = MultiHDD::locate($c->{upload_dir}, "$dx/$f->{file_real}");
   my $file_path = "$c->{upload_dir}/$src_disk_id/$dx/$f->{file_real}";

   return { error => 'No file_real specified' } if !$f->{file_real};
   return { error => 'No file' } if ! -e $file_path;

   my $dst_disk_id = MultiHDD::selectUploadDisk($c->{upload_dir});
   my $tempdir = File::Temp::tempdir(CLEANUP => 1, DIR => "$c->{temp_dir}/$dst_disk_id");

   my $ext = lc($1) if $f->{file_name} =~ /\.(zip|rar|7z)/;
   return({ error => "Unknown archive type: $f->{file_name}" }) if !$ext;

   my $prog = $ext eq 'rar' ? 'rar' : '7za';
   my $files = $f->{extract_files} || $f->{files};
   my @files = map { _sanitize($_) } @{ JSON::decode_json($files) } if $files;

   my @password = "-p$f->{extract_password}" if $f->{extract_password};
   my @output_dir = $prog eq 'rar' ? "$tempdir/" : "-o$tempdir/";
   my $extract_results = saferun($prog, $f->{delete} ? "d" : "x", $file_path, @output_dir, @password, @files);

   if($extract_results->exit_code > 0) {
      return { error => "Failed while extracting" };
   }

   if($f->{split}) {
      my $new_tempdir = File::Temp::tempdir(CLEANUP => 1, DIR => $c->{temp_dir});
      my $split_results = saferun($prog, 'a', "-v$f->{split}", "$new_tempdir/" . _sanitize($f->{file_name}) . ".part.$ext", "-ep1", "$tempdir/");

      return { error => "Failed while splitting" } if $split_results->exit_code != 0;

      # Make further operations working on the top of split compression results
      $tempdir = $new_tempdir;
   }

   if($f->{compress_back}) {
      my $compress_results = saferun($prog, "a", "-ow", "$file_path.rar", "-ep1", "$tempdir/");
      rename("$file_path.rar", $file_path);
      return { error =>  "Failed while compressing" } if $compress_results->exit_code != 0;
   }

   if($f->{import}) {
      XUpload::ImportDir($tempdir, usr_id => $f->{usr_id}||0, disk_id => $dst_disk_id);
   }

   return {
      status => "OK",
      file_spec => XUpload::rarGetInfo($file_path, file_name_orig => $f->{file_name}),
      file_size => -s $file_path,
   };
}

sub _sanitize {
   my ($p) = @_;
   $p =~ s/^-*//g;

   my @chunks = grep { $_ !~ /^(\.|\.\.)$/} split(/\/+/, $p);
   return join('/', @chunks);
}

1;
