package Engine::MainRPC::SaveFile;
use strict;
use vars qw($ses $db $f);

use XFileConfig;

sub main
{
   my ($ctx) = @_;
   my $user = $ctx->{user};
   my $server = $ctx->{server};
   my $session = $ctx->{session};

   my $size  = $f->{file_size}||0;
   my $filename = $f->{file_name};
   my $descr = $f->{file_descr}||'';

   $filename =~ s/([^&])#/$1/g;

   my $approved_affiliate = 1 if $user && $user->{usr_aff_enabled};

   return "0:0:0:uploads are not enabled for your account type" if !$c->{upload_on} && !$approved_affiliate;

   unless($f->{no_limits})
   {
      return "0:0:0:file is too big" if $c->{max_upload_filesize} && $size>$c->{max_upload_filesize}*1024*1024;

      $c->{disk_space} = $user->{usr_disk_space} if $user && $user->{usr_disk_space};

      return "0:0:0:not enough disk space on your account" if $c->{disk_space} && $user->{total_size}+$size > $c->{disk_space}*1048576;

      if($c->{fnames_not_allowed})
      {
          my @segments = split(/\|+/, $1) if $c->{fnames_not_allowed} =~ /\((.*)\)/;
          $filename =~ s/\Q$_\E/_/gi for @segments;
      }

      if($f->{rslee})
      {
         return "0:0:0:Remote upload is not allowed for you" if !$c->{remote_url};
         if($c->{max_rs_leech})
         {
            $c->{max_rs_leech} = $user->{usr_max_rs_leech} if $user;
            my $leech_left = $c->{max_rs_leech}*1048576 - $db->SelectOne("SELECT SUM(size) FROM IP2RS WHERE created>NOW()-INTERVAL 24 HOUR AND (usr_id=? OR ip=?)",$user->{usr_id},$f->{file_ip});

            return "0:0:0:You've used all Remote traffic today" if $leech_left <= 0;
         }
         $db->Exec("INSERT INTO IP2RS SET usr_id=?, ip=?, size=?",$user->{usr_id},$f->{file_ip},$size);
      }

      return "0:0:0:unallowed extension" if ($c->{ext_allowed} && $filename!~/\.($c->{ext_allowed})$/i) || ($c->{ext_not_allowed} && $filename=~/\.($c->{ext_not_allowed})$/i);

      if(my $ban = $db->SelectRow("SELECT * FROM Bans WHERE ip=?", $f->{file_ip}))
      {
         my $ret = "0:0:0:your IP was banned by system administrator";
         $ret .= " for the reason: $ban->{reason}" if $ban->{reason};
         return $ret;
      }
   }

   $filename=~s/[^\w\d\.-]/_/g if $c->{sanitize_filename};
   $filename=~s/\.(\w+)$/"$c->{add_filename_postfix}\.$1"/e if $c->{add_filename_postfix};

   my $usr_id = $user ? $user->{usr_id} : 0;
   $usr_id = $session->{view_as} if $user && $user->{usr_adm} && $session->{view_as};
   my $fld_id = $f->{fld_path} ? &CheckFolder($f->{fld_path}, $usr_id) : $f->{fld_id};

   if($f->{fld_path} && $f->{check_login} && $user)
   {
      # Handle WebDAV file rewriting
      my $existing_file = $db->SelectRow("SELECT * FROM Files WHERE usr_id=? AND file_fld_id=? AND file_name=?", $user->{usr_id}, $fld_id||0, $filename);
      if($existing_file)
      {
         print STDERR "Rewrite file: user=$user->{usr_login}, path=$f->{fld_path}, filename=$filename\n";
         $ses->DeleteFile($existing_file);
      }
   }
   
   my $md5 = $f->{file_md5}||'';
   my $code = $ses->randchar(12);
   while($db->SelectOne("SELECT file_id FROM Files WHERE file_code=? OR file_real=?",$code,$code)){$code = $ses->randchar(12);}
   my $del_id = $ses->randchar(10);
   
   return "0:0:0:this file is banned by administrator" if $db->SelectOne("SELECT id FROM Reports WHERE ban_size=? AND ban_md5=? LIMIT 1",$size,$md5);

   if($c->{up_limit_days} && $ses->getUserLimit('up_limit', user => $user))
   {
      my $cond = $user && $user->{usr_id} ? "usr_id=".$user->{usr_id} : "file_ip='$f->{file_ip}'";
      my $uploaded_last = $db->SelectOne("SELECT SUM(file_size) / POW(2,20)
            FROM Files
            WHERE $cond
            AND file_created > NOW() - INTERVAL ? DAY",
            $c->{up_limit_days});
      return "0:0:0:upload limit exceeded" if $uploaded_last + ($size / 2**20) > $ses->getUserLimit('up_limit', user => $user);
   }
   $db->SelectOne("SELECT SUM(file_size) FROM Files WHERE usr_id=? AND file_created > NOW() - INTERVAL 1 DAY");
   
   my $ex = $db->SelectRow("SELECT * FROM Files WHERE file_size=? AND file_md5=? AND file_real_id=0 LIMIT 1",$size,$md5)
            if $c->{anti_dupe_system};

   my $real = $ex->{file_real} if $ex;
   my $real_id = $ex->{file_id} if $ex;
   my $srv_id = $ex ? $ex->{srv_id} : $server->{srv_id};
   my $file_size_encoded = $ex ? $ex->{file_size_encoded} : 0;
   $f->{file_spec}=$ex->{file_spec} if $ex;
   #$server->{srv_id} = $ex->{srv_id} if $ex;
   $real ||= $code;

   my $file_awaiting_approve = 1 if $c->{files_approve};
   $file_awaiting_approve = 0 if $c->{files_approve_regular_only} && $user && $user->{usr_aff_enabled};
   
   $db->Exec("INSERT INTO Files 
              SET file_name=?, usr_id=?, srv_id=?, file_descr=?, file_fld_id=?, file_public=?, file_adult=?, file_code=?, file_real=?, file_real_id=?, file_del_id=?, file_size=?, file_size_encoded=?,
                  file_password=?, file_ip=?, file_md5=?, file_spec=?, file_awaiting_approve=?, file_upload_method=?, file_premium_only=?, file_created=NOW(), file_last_download=NOW()",
               $filename,
               $usr_id,
               $srv_id,
               $descr,
               $fld_id||0,
               $f->{file_public}||0,
               $f->{file_adult}||0,
               $code,
               $real,
               $real_id||0,
               $del_id,
               $size,
               $file_size_encoded,
               $f->{file_password}||'',
               $f->{file_ip}||'1.1.1.1',
               $md5,
               $f->{file_spec}||'',
               $file_awaiting_approve||0,
               $f->{file_upload_method}||'',
               $c->{file_prem_only_default}||'',
             );
   
   my $file_id = $db->getLastInsertId;

   $ses->ApplyHooks('fs.save_file', $file_id);

   $size=0 unless $code eq $real;
   $db->Exec("UPDATE Servers 
              SET srv_files=srv_files+1, 
                  srv_disk=srv_disk+?, 
                  srv_last_upload=NOW() 
              WHERE srv_id=?", $size, $srv_id );
   
   $db->Exec("INSERT INTO Stats SET day=CURDATE(), uploads=1 ON DUPLICATE KEY UPDATE uploads=uploads+1");

   if($session && $session->{api_key_id})
   {
      $db->Exec("INSERT INTO APIStats SET key_id=?, day=CURDATE(),
         uploads=1, bandwidth_in=?
         ON DUPLICATE KEY UPDATE uploads=uploads+1, bandwidth_in=bandwidth_in+?",
         $session->{api_key_id}, $size, $size);
   }
   
   my $file = $db->SelectRow("SELECT * FROM Files WHERE file_id=?", $file_id);

   if($c->{m_e} && $file->{file_name} =~ /\.(avi|divx|xvid|mpg|mpeg|vob|mov|3gp|flv|wmv|mkv)$/i)
   {
      $db->Exec("INSERT IGNORE INTO QueueEncoding
           SET file_real=?,
               file_id=?,
               srv_id=?,
               created=NOW()",
           $file->{file_real},
           $file->{file_id},
           $file->{srv_id});
   }

   if($f->{compile})
   {
      my $link = $ses->makeFileLink($file);
      my $del_link="$link?killcode=$del_id";
      return "$file_id:$code:$real:OK=$link|$del_link";
   }
   
   return "$file_id:$code:$real:OK";
}

sub CheckFolder
{
   my ($path, $usr_id) = @_;
   return if $path =~ /^(\.|\.{2})$/;

   my @path = split(/\/+/, $path);
   die("No usr_id") if !$usr_id;

   my $fld_id;
   my $parent_id = 0;
   for(@path)
   {
      next if $_ =~ /^\s*$/;
      if(!($fld_id = $db->SelectOne("SELECT fld_id FROM Folders WHERE fld_name=? AND usr_id=? AND fld_parent_id=?", $_, $usr_id, $parent_id)))
      {
         $db->Exec("INSERT INTO Folders SET fld_name=?, usr_id=?, fld_parent_id=?", $_, $usr_id, $parent_id);
         $fld_id = $db->getLastInsertId;
      }
      $parent_id = $fld_id;
   }

   return $fld_id;
}

1;
