package Engine::WebDAV::Response::PropStat;
use strict;
use base 'Engine::WebDAV::Response';
use Engine::Extras::PathUtils;

sub new
{
   my ($class) = @_;
   my $self = $class->SUPER::new();
   $self->{'xmlns:D'} = "DAV:";
   $self->{'D:response'} = [];
   return $self;
}

sub add_dir
{
   my ($self, $object) = @_;
   $self->add_response({ %$object, dir => 1 });
}

sub add_file
{
   my ($self, $object) = @_;
   $self->add_response({ %$object, dir => 0 });
}

sub add_response
{
   my ($self, $object) = @_;

   my $response = {};
   $response->{'xmlns:lp1'} = "DAV:";
   $response->{'xmlns:lp2'} = "http://apache.org/dav/props/";
   $response->{'D:href'} = [ Engine::Extras::PathUtils::normalize_path($object->{path}) ];

   if($object->{created})
   {
      my ($year,$month,$day,$hour,$minute,$second) = _parse_mysql_date($object->{created});
      $response->{'D:propstat'}->{'D:prop'}->{'lp1:creationdate'} = [ "$year-$month-$day\T$hour:$minute:$second\Z" ];
   }

   print STDERR "size=$object->{size}\n";

   $response->{'D:propstat'}->{'D:prop'}->{'lp1:getcontentlength'} = [ $object->{size} ] if $object->{size};
   $response->{'D:propstat'}->{'D:prop'}->{'lp1:resourcetype'} = [ $object->{dir} ? { 'D:collection' => [''] } : '' ];
   $response->{'D:propstat'}->{'D:status'} = ['HTTP/1.1 200 OK'];

   push @{ $self->{'D:response'} }, $response;

   return $self;

}

sub _parse_mysql_date
{
   return split(/[-:\s]/, shift);
}

1;
