package CheckParams;
use strict;
use Exporter;
@CheckParams::ISA    = qw(Exporter);
@CheckParams::EXPORT = qw(check_params copy_options);

=pod
Performs function parameters check and throws exception if any errors found:
   check_params($hashref, qw(*required_param1 *required_param2 optional_param1 optional_param2));
=cut

sub check_params {
   my ($hashref, @known_opts) = @_;
   my ($pkg, $fn, $ln) = caller(1);

   die("Param not specified, should be a hashref at $fn:$ln\n") if ref($hashref) ne 'HASH';
   die("Param is not a hashref at $fn:$ln\n") if ref($hashref) ne 'HASH';

   for(@known_opts) {
      die("Required option: $_ at $fn:$ln\n") if s/\*// && !defined($hashref->{$_});
   }

   my %known_opts = map { $_ => 1 } @known_opts;

   for(keys(%$hashref)) {
      die("Unknown option: $_ at $fn:$ln\n") if !$known_opts{$_};
   }
}

sub copy_options {
   my ($hashref, @opts) = @_;
   my ($pkg, $fn, $ln) = caller(1);

   return {
      map {
         die("Required option: $_ at $fn:$ln\n") if s/\*// && !defined($hashref->{$_});
         $_ => $hashref->{$_}
      } @opts
   };
}

1;
