package Plugins::Payments::authorize;
use Payments;
use Digest::HMAC_MD5 qw(hmac_md5_hex);
use Digest::MD5 qw(md5_hex);
use base 'Payments';
use vars qw($ses $c);
use strict;

sub options {
	return {
		name=>'authorize', title=>'Authorize.net', account_field=>'authorize_login_id', image=>'buy_authorize.gif', 
		s_fields=>[
			{title=>'Your Authorize.net API Login', name=>'authorize_login_id', type=>'text', comment=>'optional'},
			{title=>'Your Authorize.net API Key', name=>'authorize_secret', type=>'text', comment=>'optional'},
			]
		};
}

sub checkout {
	my ($self, $f) = @_;
   return if $f->{type} ne 'authorize';
   my $sequence	= int(rand(1000));
   my $timeStamp	= time();
   my $fingerprint = hmac_md5_hex($c->{authorize_login_id} . "^" . $sequence . "^" . $timeStamp . "^" . $f->{amount} . "^", $c->{authorize_secret});
   print"Content-type:text/html\n\n";
   print <<END
<HTML><BODY onLoad="document.F1.submit();">
<FORM name="F1" method='post' action='https://secure.authorize.net/gateway/transact.dll' >
<INPUT type='hidden' name='x_login' value='$c->{authorize_login_id}' />
<INPUT type='hidden' name='x_amount' value='$f->{amount}' />
<INPUT type='hidden' name='x_description' value='$c->{site_name} Premium Account' />
<INPUT type='hidden' name='x_invoice_num' value='$f->{id}' />
<INPUT type='hidden' name='x_fp_sequence' value='$sequence' />
<INPUT type='hidden' name='x_fp_timestamp' value='$timeStamp' />
<INPUT type='hidden' name='x_fp_hash' value='$fingerprint' />
<INPUT type='hidden' name='x_show_form' value='PAYMENT_FORM' />
<INPUT type='hidden' name='x_receipt_link_method' value='LINK' />
<INPUT type='hidden' name='x_receipt_link_text' value='Proceed to shop' />
<INPUT type='hidden' name='x_receipt_link_url' value='$c->{site_url}/?payment_complete=$f->{id}-1' />
<INPUT type='hidden' name='x_cancel_url' value='$c->{site_url}/?op=payments' />
<input type='submit' value="OK"/>
</FORM>
</BODY></HTML>
END
;
	exit;
}

sub verify {
	my ($self, $f) = @_;
   return if !$f->{x_invoice_num};
	my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{x_invoice_num}) || die("Transaction not found: '$f->{x_invoice_num}'");
   my $sig = md5_hex(join('', $c->{dl_key}, $c->{authorize_login_id}, $f->{x_trans_id}, $f->{x_amount}));
   die("Wrong signature") if $sig ne lc($f->{x_MD5_Hash});
   die("Wrong amount") if $transaction->{amount} != $f->{x_amount};
   die("Wrong status") if $f->{x_response_code} != 1;
	$f->{payer_email} = $f->{x_email};
   return($transaction);
}

1;
