package Plugins::Payments::authorize_se;

use Payments;
use strict;
use base 'Payments';
use vars qw($ses $c);
use Digest::MD5 qw(md5_hex);
use URI::Escape;

sub options {
   return {
      name=>'authorize_se', title=>'Credit Cards', account_field=>'authorize_se_vendor_id', image=>'cc2.png', reseller_disabled => '1',
      weight => '3',
         s_fields=>[
            {title=>'Your Authorize.se Vendor ID', name=>'authorize_se_vendor_id', type=>'text', comment=>'optional'},
            {title=>'Your Authorize.se Site ID', name=>'authorize_se_site_id', type=>'text', comment=>'optional'},
            {title=>'Your Authorize.se Plans', name=>'authorize_se_plans', type=>'text', comment=>'optional', size=>56},
            {title=>'Your Authorize.se URL', name=>'authorize_se_url', type=>'text', comment=>'optional'},
            {title=>'Your Authorize.se Secret', name=>'authorize_se_secret', type=>'text', comment=>'optional'},
            ],
      };
}

sub checkout {
   my ($self, $f) = @_;
   return if $f->{type} ne 'authorize_se';

   my $i = 0;
   my $days = &ParsePlans($c->{payment_plans}, 'hash')->{$f->{amount}};
   my $authorize_plans = &ParsePlans($c->{authorize_se_plans}, 'array');
   my ($p) = grep { $_->{key} == $days  } @$authorize_plans;
   return $ses->message("Selected plan isn't available for payment method in use") if !$p;

   my $email = uri_escape($ses->getUser->{usr_email}) if $ses->getUser;
   return "$c->{authorize_se_url}?v=$c->{authorize_se_vendor_id}&s=$c->{authorize_se_site_id}&p=$p->{value}&c=$f->{id}";
}

sub verify {
   my ($self, $f) = @_;
   return if !$f->{PAYMENT_ID};

   $f->{txn_id} = $f->{PAYMENT_ID};
   $f->{out} = "SUCCESS";
   my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?", $f->{CUSTOM_VARIABLE}) || error("Transaction not found: '$f->{CUSTOM_VARIABLE}'");
   &revokePremium($transaction) if $f->{CALLBACK_ACTION} eq 'DELETE';
   error("Resubmitting detected") if $transaction->{verified} && $f->{PAYMENT_ID} eq $transaction->{txn_id};
   my $sig = md5_hex(join(" ", $c->{authorize_se_secret}, $f->{CALLBACK_ACTION}, $f->{PAYMENT_ID}, $f->{CUSTOM_VARIABLE}));
   error("Wrong status") if $f->{CALLBACK_ACTION} ne 'ADD';
   error("Wrong signature") if uc($sig) ne uc($f->{MD5_VALIDATION});
   error("Wrong amount") if $f->{AMOUNT} < $transaction->{amount};
   $ses->db->Exec("UPDATE Transactions SET report_awaiting=1, last_rebill_date=NOW() WHERE id=?", $transaction->{id});
   $f->{txn_id} = $f->{PAYMENT_ID};
   return $transaction;
}

sub revokePremium
{
   my ($transaction) = @_;
   error("Already refunded") if $transaction->{refunded};
   $ses->db->Exec("UPDATE Transactions SET refunded=1 WHERE id=?", $transaction->{id});
   print STDERR "Revoking $transaction->{days} days premium from $transaction->{usr_id}\n";
   $ses->db->Exec("UPDATE Users SET usr_premium_expire=usr_premium_expire - INTERVAL ? DAY WHERE usr_id=?",
      $transaction->{days},
      $transaction->{usr_id});
   my $payments = $ses->db->SelectARef("SELECT * FROM PaymentsLog WHERE transaction_id=?", $transaction->{id});
   for(@$payments)
   {
      print STDERR "Revoking \$$_->{amount} from affiliate $_->{usr_id_to}\n";

      $ses->db->Exec("UPDATE Users SET usr_money=usr_money-? WHERE usr_id=?",
         $_->{amount}||0, $_->{usr_id_to});

      $ses->db->Exec("INSERT INTO Stats2
                      SET usr_id=?, day=CURDATE(), refunds_amount=?
            ON DUPLICATE KEY UPDATE refunds_amount=refunds_amount+?",
            $_->{usr_id_to},
            $_->{amount},
            $_->{amount});
   }

   print "Content-type: text/html\n\nSUCCESS";
   exit();
}

sub error
{
   print STDERR "$_[0]\n";
   print "Content-type: text/html\n\nSUCCESS";
   exit();
}

sub ParsePlans
{
   my ($str, $ref) = @_;
   $ref ||= 'array';
   my ($ref_array, $ref_hash, $ref_hash_reverse) = ([], {}, {});
   for ( split( /,/, $str ) ) {
      /([\d\.]+)=(.*)/;
      my $per_day = sprintf('%.02f',$1/$2) if $2 > 0;
      my $obj = { amount => $1, days => $2, key => $1, value => $2, site_url => $c->{site_url}, per_day=>$per_day };
      push @$ref_array, $obj;
   }
   foreach(@$ref_array) {
      my $amount = sprintf("%.02f",$_->{amount});
      $ref_hash->{$amount} = $_->{value};
      $ref_hash_reverse->{$_->{value}} = $amount;
   }
   return($ref_array) if $ref eq 'array';
   return($ref_hash) if $ref eq 'hash';
   return($ref_hash_reverse) if $ref eq 'hash_reverse';
}

1;
