###############################################################################
# G2A Pay gateway plugin for XFileSharing
# version: 0.1.13 beta
# author: Gabriel Zima G2A.COM (integration@g2a.com)
###############################################################################

package Plugins::Payments::g2apay;
use Payments;
use base 'Payments';
use vars qw($ses $c);
use Digest::SHA qw(sha256_hex);

sub options {
	return {
		name=>'g2apay', title=>'G2A Pay', account_field=>'g2a_merchant_email', image=>'buy_g2a.png',
			s_fields=>[
				{title=>'Your G2A Merchant e-mail', name=>'g2a_merchant_email', type=>'text', size=>24, comment=>'required'},
				{title=>'Your G2A API Hash', name=>'g2apay_api_hash', type=>'text', size=>40, comment=>'required'},
				{title=>'Your G2A API Secret', name=>'g2apay_secret', type=>'text', size=>40, comment=>'required'}
			],
		};
}

sub checkout {
	my ($self, $f) = @_;
	
	#calculate hash
    my $hash = sha256_hex($f->{id} . $f->{amount} . $c->{currency_code} . $c->{g2apay_secret});
	return if $f->{type} ne 'g2apay';
	
	# generate HTML
	print "Content-type:text/html\n\n";
	print <<OUT
		<html>
		<head>
		<title>Redirect to G2A Pay</title>
		<script src="https://code.jquery.com/jquery-1.10.2.js"></script>
		</head>
		<body onLoad="\$('#createQuote').submit();">
		<div align="center" style="height:150px;">You are being redirected to G2A Pay, please dont refresh the page...</div>
		<div align="center"><img src="http://preloaders.net/preloaders/252/Three%20blocks.gif"></div>
		<form method="POST" id="createQuote" action="https://checkout.pay.g2a.com/index/createQuote">
		<input type="hidden" name="email" value="$f->{payer_mail}">
		<input type="hidden" name="api_hash" value="$c->{g2apay_api_hash}">
		<input type="hidden" name="hash" value="$hash">
		<input type="hidden" name="amount" value="$f->{amount}">
		<input type="hidden" name="currency" value="$c->{currency_code}">
		<input type="hidden" name="items" value="{}">
		<input type="hidden" name="description" value="">
		<input type="hidden" name="order_id" value="$f->{id}">
		<input type="hidden" name="url_failure" value="$c->{site_url}/?op=payments">
		<input type="hidden" name="url_ok" value="$c->{site_url}/?payment_complete=$f->{id}-0">
		</form>
		<script>\$(function(){\$("#createQuote").submit(function(t){t.preventDefault();var e=t.currentTarget.action;\$.ajax({url:e,type:"post",dataType:"json",data:\$("#createQuote").serialize(),success:function(t){var e="https://checkout.pay.g2a.com/index/gateway?token="+t.token;\$(location).attr("href",e)}})})});</script>
		</body>
		</html>
OUT
		;
	exit();
}

sub verify {
	my ($self, $f) = @_;
	return if !$f->{userOrderId};
	
	my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{userOrderId}) || die( "Transaction not found: '$f->{userOrderId}'"  );
	
	my $g2apay_secret = $c->{g2apay_secret};
	$g2apay_secret =~ s/&lt;/</g;
	$g2apay_secret =~ s/&gt;/>/g;
	my $sign = sha256_hex($f->{transactionId} . $f->{userOrderId} . $f->{amount} . $g2apay_secret);

	print STDERR "sign=$sign\n";
	
	die("Wrong signature") if $sign ne $f->{hash};
	die("Wrong amount") if $f->{amount} != $transaction->{amount};
	die("Wrong status") if $f->{status} ne 'complete';
	
	return($transaction);
}

1;
