package Plugins::Payments::lavapay;
use Payments;
use base 'Payments';
use vars qw($ses $c);
use URI;
use URI::QueryParam;
use LWP::UserAgent;

sub options {
   return {
      name=>'lavapay',
      title=>'LavaPay',
      account_field=>'lavapay_wallet_id',
      image=>'buy_lavapay.png',
      listed_reseller=>'1',
         s_fields=>[
            {title=>'Your LavaPay Wallet ID', name=>'lavapay_wallet_id', type=>'text', size => 16},
            {title=>'Your LavaPay Receiver E-Mail', name=>'lavapay_receiver_email', type=>'text', size => 16},
            {title=>'Your LavaPay API Key', name=>'lavapay_api_key', type=>'text', comment=>'', size => 32},
            ],
      };
}

sub checkout {
   my ($self, $f) = @_;
   return if $f->{type} ne 'lavapay';

   my $email = $ses->getUser->{usr_email} if $ses->getUser;
   
   my $uri = URI->new('https://www.LavaPay.com/checkout/');
   $uri->query_param(LavaReceiver => $c->{lavapay_receiver_email});
   $uri->query_param(LavaItem1Name => "$c->{site_name} premium account");
   $uri->query_param(LavaItem1Price => $f->{amount});
   $uri->query_param(LavaCurrency => $c->{currency_code});
   $uri->query_param(LavaBuyerEmail => $f->{email} || $email);
   $uri->query_param(LavaInvoice => $f->{id});

   return $uri->as_string;
}

sub verify {
   my ($self, $f) = @_;
   return if !$f->{LavaCallbackId};

   my $transaction = $db->SelectRow("SELECT * FROM Transactions WHERE id=?", $f->{LavaInvoice}) || die("No such transaction");

   my $ua = LWP::UserAgent->new;
   my $res = $ua->post('http://www.lavapay.com/callback-verify.html', {
      %{$f},
      LavaVerify => true,
   });

   print STDERR "Received from LavaPay: ", $res->decoded_content, "\n";

   die("Invalid receiver e-mail") if $f->{LavaReceiverEmail} ne $c->{lavapay_receiver_email};
   die("Test mode") if $f->{LavaCallbackTest};
   die("Wrong amount") if $f->{LavaItem1Price} != $f->{amount};
   die("Wrong currency") if $f->{LavaTxnCurrency} ne $c->{currency_code};
   die("Wrong status") if $f->{LavaTxnStatus} ne 'Completed';

   return $transaction;
}

1;
