package Plugins::Payments::paygol;
use Payments;
use base 'Payments';
use vars qw($ses $c);
use strict;

use LWP::UserAgent;
use Geo::IP2;
use XML::Simple;

sub options
{
   return
   {
      name => 'paygol', title  =>  'PayGol',
      account_field => 'paygol_service_id',
      submethods => [
         { name => 'creditcard', title => 'Credit Card', image => 'buy_cc1.png',        listed_reseller => '1' },
         { name => 'sofort',     title => 'Sofort',      image => 'buy_sofort.png',     listed_reseller => '1' },
         { name => 'ideal',      title => 'iDeal',       image => 'buy_ideal.png',      listed_reseller => '1' },
         { name => 'boleto',     title => 'Boleto',      image => 'buy_boleto.png',     listed_reseller => '1' },
         { name => 'santander',  title => 'Santander',   image => 'buy_santander.png',  listed_reseller => '1' },
      ],
      s_fields => [
         { title => 'Your PayGol Service ID', name => 'paygol_service_id', type => 'text',size => 15 },
         { title => 'Your PayGol Secret Key', name => 'paygol_secret', type => 'text', size => 30 },
      ]
   };
}

sub checkout
{
   my ($self, $f) = @_;
   return if $f->{type} ne 'paygol';
   return $ses->message("Subpayment method not specified") if !$f->{submethod};
   return $ses->message("Invalid subpayment method: $f->{submethod}") if $f->{submethod} !~ /^(creditcard|sofort|ideal|bancontact|przlewy24|boleto|santander)$/;

   my $gi      = Geo::IP2->new("$c->{cgi_path}/GeoLite2-Country.mmdb");
   my $country = $gi->country_code_by_addr( $ses->getIP );

   my $ua = LWP::UserAgent->new(agent => 'XFS-Agent');
   push @{ $ua->requests_redirectable }, 'POST';
   my $res = $ua->post("https://www.paygol.com/pay", {
      pg_serviceid => $c->{paygol_service_id},
      pg_currency => $c->{currency_code},
      pg_price => $f->{amount},
      pg_custom => $f->{id},
      pg_return_url => "$c->{site_url}/?payment_complete=$f->{id}-1",
      pg_cancel_url => "$c->{site_url}/?op=payments",
      pg_mode => "api",
      pg_country => $country,
      pg_format => "xml",
      pg_ip => $ses->getIP,
      pg_email => $f->{email},
      pg_method => $f->{submethod},
      pg_first_name => $f->{first_name},
      pg_last_name => $f->{last_name},
   });

   $ses->AdminLog("Received from PayGol: " . $res->as_string());

   my $ret = XMLin($res->decoded_content);
   return $ses->message($ret->{error_message}) if !$ret->{payment_method_url};
   return $ses->message("Internal error") if !$ret->{payment_method_url};
   return $ret->{payment_method_url};
}

sub error
{
   print STDERR "$_[0]\n";
   print "Content-type: text/plain\n\nRDI_RESULT=OK";
   exit;
}

sub verify
{
   my ($self, $f) = @_;
   return if !$f->{frmprice};

   my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{custom}) || error( "Transaction not found: '$f->{custom}'"  );
   error("Wrong amount") if $f->{frmprice} != $transaction->{amount};
   error("Wrong currency") if $f->{frmcurrency} ne $c->{currency_code};

   return($transaction);
}

1;
