package Plugins::Payments::payo;
use Payments;
use strict;
use base 'Payments';
use vars qw($ses $c);

sub options {
   return {
      name=>'payo', title=>'Payo', account_field=>'payo_member_id', image=>'buy_cc1.png', weight => 2,
         s_fields=>[
            {title=>'Your Payo Member ID', name=>'payo_member_id', type=>'text', comment=>'optional'},
            {title=>'Your Payo Products', name=>'payo_products', type=>'text', comment=>'optional', size=>56},
            {title=>'Your Payo Secret', name=>'payo_secret', type=>'text', comment=>'optional'},
            ],
      };
}

sub checkout {
   my ($self, $f) = @_;
   return if $f->{type} ne 'payo';

   my $plans = $ses->ParsePlans($c->{payment_plans}, 'array');
   my $plans_payo = $ses->ParsePlans($c->{payo_products}, 'array');
   die("Payo plans are not matching with global plans") if int(@$plans) != int(@$plans_payo);

   my $i = 0;
   my ($plan) = grep { $plans->[$i++]->{amount} == $f->{amount} } @$plans_payo;
   die("No such plan found") if !$plan;

   my $tax = $plan->{amount} - $f->{amount};
   my $product_id = $plan->{value};
   die("No such product id") if !$product_id;

   my $days = $ses->ParsePlans($c->{payment_plans}, 'hash')->{$f->{amount}};
   my $usr_id = $ses->getUserId;
   my $is_subscription = length($product_id) == 2; # poor practice

   $ses->db->Exec("UPDATE Transactions SET brutto_amount=? WHERE id=?", $plan->{amount}, $f->{id});

   print"Content-type:text/html\n\n";
   if($is_subscription)
   {
      print<<END
<HTML><BODY>
<form name="F1" method=post action=https://payo.io/subscription>
<input type=hidden name=member value="$c->{payo_member_id}">
<input type=hidden name=action value="subscription">
<input type=hidden name=orderid value="$f->{id}">
<input type=hidden name=name value="$c->{site_name} $days days subscription + fee">
<input type=hidden name=id value="$product_id">
<input type=hidden name=price value="$plan->{amount}">
<input type=hidden name=period value="$days">
<input type=hidden name=tax value="">
<input type=hidden name=ureturn value="$c->{site_url}/?payment_complete=$f->{id}-$usr_id">
<input type=hidden name=unotify value="$c->{site_cgi}/ipn.cgi">
<input type=hidden name=ucancel value="$c->{site_url}/?op=payments">
<input type=hidden name=comments value="">
<input type="submit" value="Redirecting...">
</form>
</BODY></HTML>
END
;
   }
   else
   {
      print<<END
<HTML><BODY onLoad="document.F1.submit();">
<form name="F1" method=post action=https://payo.io/checkout>
<input type=hidden name=member value="$c->{payo_member_id}">
<input type=hidden name=action value="product">
<input type=hidden name=orderid value="$f->{id}">
<input type=hidden name=name value="$c->{site_name} $days days premium account + fee">
<input type=hidden name=id value="$product_id">
<input type=hidden name=price value="$plan->{amount}">
<input type=hidden name=tax value="">
<input type=hidden name=ureturn value="$c->{site_url}/?payment_complete=$f->{id}-$usr_id">
<input type=hidden name=unotify value="$c->{site_cgi}/ipn.cgi">
<input type=hidden name=ucancel value="$c->{site_url}/?op=payments">
<input type=hidden name=comments value="">
<input type="submit" value="Redirecting...">
</form>
</BODY></HTML>
END
;
   }
   exit;
}

sub verify {
   my ($self, $f) = @_;
   return if !$f->{subname} && !$f->{pname};
   my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{orderid}) || error( "Transaction not found: '$f->{orderid}'"  );
   die("Wrong amount") if ($f->{price}||$f->{total}) != $transaction->{brutto_amount};
   die("Wrong status") if $f->{status} ne 'Transaction Success';
   return $transaction;
}

sub error {
   print STDERR "$_[0]\n";
   print "Content-type: text/plain\n\n";
   exit;
}

1;
