
# Connection: Keep-Alive
# User-Agent: PHP-SOAP/5.3.6-13ubuntu3.10
# Content-Type: text/xml; charset=utf-8
# Content-Length: 742

package Plugins::Payments::paysafecard;
use Payments;
use base 'Payments';
use LWP::UserAgent;
use XML::Simple;
use Data::Dumper;
use JSON;
use vars qw($ses $db $c);
use strict;


sub options {
   return {
      name=>'paysafecard', title=>'PaySafeCard', account_field=>'paysafecard_username', image=>'buy_paysafecard.gif', 
      s_fields=>[
         {title=>'Your PaySafeCard Username', name=>'paysafecard_username', type=>'text'},
         {title=>'Your PaySafeCard Password', name=>'paysafecard_password', type=>'text'},
         ]
      };
}

sub checkout {
   my ($self, $f) = @_;
   return if $f->{type} ne 'paysafecard';
   return $ses->message("Only EUR currency is supported") if $c->{currency_code} ne 'EUR';
   my $usr_id = $ses->getUser ? $ses->getUserId : 0;
   my $ip = $ses->getIP;
   my $soap_res = soapRequest('createDisposition', <<BLOCK
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="urn:pscservice">
<SOAP-ENV:Body>
   <ns1:createDisposition>
      <ns1:username>$c->{paysafecard_username}</ns1:username>
      <ns1:password>$c->{paysafecard_password}</ns1:password>
      <ns1:mtid>$f->{id}</ns1:mtid>
      <ns1:subId></ns1:subId>
      <ns1:amount>$f->{amount}</ns1:amount>
      <ns1:currency>$c->{currency_code}</ns1:currency>
      <ns1:okUrl>$c->{site_url}/?payment_complete=$f->{id}-1</ns1:okUrl>
      <ns1:nokUrl>$c->{site_url}/?op=payments</ns1:nokUrl>
      <ns1:merchantclientid>$usr_id</ns1:merchantclientid>
      <ns1:pnUrl>$c->{site_cgi}/ipn.cgi</ns1:pnUrl>
      <ns1:clientIp>$ip</ns1:clientIp>
      <ns1:shopId></ns1:shopId>
      <ns1:shopLabel></ns1:shopLabel>
   </ns1:createDisposition>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
BLOCK
);
   if($soap_res->{'ns1:errorCode'}) {
      print STDERR "Error " . $soap_res->{'ns1:errorCode'};
      $ses->message("Transaction could not be initiated due to connection problems.<br>
      If the problem persists, please contact our support.");
   }
   $soap_res->{"ns1:amount"} = $f->{amount};
   $soap_res->{"ns1:currency"} = $c->{currency_code};
   $soap_res->{"ns1:subId"} = '';
   my @keys = ('mid', 'mtid', 'amount', 'currency', 'subId');
   my $query = buildQuery($soap_res, 'ns1', @keys);
   $ses->redirect("https://customer.cc.at.paysafecard.com/psccustomer/GetCustomerPanelServlet?$query");
}

sub verify {
   my ($self, $f) = @_;
   return if !$f->{mtid};
   my $transaction = $db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{mtid}) || die( "Transaction not found_paysafe: '$f->{mtid}'");
   # Note: don't try to parse the serial numbers here, as sometimes PaySafeCard provides the currencies that are other than EUR
   my $soap_res = soapRequest('executeDebit', <<BLOCK
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="urn:pscservice">
<SOAP-ENV:Body>
   <ns1:executeDebit>
      <ns1:username>$c->{paysafecard_username}</ns1:username>
      <ns1:password>$c->{paysafecard_password}</ns1:password>
      <ns1:mtid>$transaction->{id}</ns1:mtid>
      <ns1:subId></ns1:subId>
      <ns1:amount>$transaction->{amount}</ns1:amount>
      <ns1:currency>$c->{currency_code}</ns1:currency>
      <ns1:close>1</ns1:close>
   </ns1:executeDebit>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
BLOCK
);
   print STDERR "Trying to perform executeDebit...\n";
   print STDERR "Received response: ", JSON::encode_json($soap_res), "\n";
   die("Wrong status") if $soap_res->{'ns1:errorCode'} != 0;
   return $transaction;
}

sub buildQuery {
   my ($object, $ns, @keys) = @_;
   return join('&', map { "$_=".$object->{"$ns:$_"} } @keys);
}

=item
It's obvious that we can't make a use of PHP SOAPClient here. So, need to do some low-level stuff
to keep the backend happy.
=cut

sub soapRequest {
   my ($action, $xml) = @_;
   my $ua = LWP::UserAgent->new(agent => 'SOAPClient', timeout => 90);
   my $soap_request = HTTP::Request->new(POST => 'https://soa.paysafecard.com/psc/services/PscService');
   $soap_request->content($xml);
   $soap_request->header(SOAPAction => $action);
   my $res = $ua->request($soap_request);
   my $ret = XMLin($res->decoded_content);
   my $soap_ret = $ret->{"soap:Body"}->{"ns1:$action"."Response"}->{"ns1:$action"."Return"};
   return $soap_ret||$ret;
}

1;
