package Plugins::Payments::paysera;
use Payments;
use base 'Payments';
use MIME::Base64;
use Digest::MD5 qw(md5_hex);
use URI::Query;
use vars qw($ses $c);
use strict;

sub options
{
   return
   {
      name=>'paysera', title=>'Paysera', account_field=>'paysera_project_id', image=>'buy_paysera.png', 
      listed_reseller=>'1',
      s_fields=>[
         {title=>'Your Paysera Project ID', name=>'paysera_project_id', type=>'text',size=>5},
         {title=>'Your Paysera Project Password', name=>'paysera_secret', type=>'text', size=>30},
         ]
   };
}

sub checkout
{
   my ($self, $f) = @_;
   return if $f->{type} ne 'paysera';

   my $payload = URI::Query->new({
      projectid => $c->{paysera_project_id},
      orderid => $f->{id},
      accepturl => "$c->{site_url}/?payment_complete=$f->{id}-1",
      cancelurl => "$c->{site_url}/?op=payments",
      callbackurl => "$c->{site_cgi}/ipn.cgi",
      version => 9,
      amount => int($f->{amount} * 100),
      currency => $c->{currency_code},
      p_email => $f->{email},
   });
   my $data = encode_base64($payload->stringify, '');
   my $sign = md5_hex($data . $c->{paysera_secret});

   return "https://www.paysera.com/pay/?data=$data&sign=$sign";
}

sub error
{
   print STDERR "$_[0]\n";
   print "Content-type: text/plain\n\nOK";
   exit;
}

sub verify
{
   my ($self, $f) = @_;
   return if !$f->{data};

   my $sign = md5_hex($f->{data} . $c->{paysera_secret});
   error("Wrong signature") if $sign ne $f->{ss1};

   my %data = URI::Query->new(decode_base64($f->{data}))->hash();
   my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?", $data{orderid}) || error( "Transaction not found: '$data{orderid}'"  );
   return $transaction;
}

1;
