package Plugins::Payments::payssion;
use Payments;
use base 'Payments';
use vars qw($ses $c);

use Digest::MD5 qw(md5_hex);
use JSON;

sub options {
   return {
      name=>'payssion', title=>'Payssion', account_field=>'payssion_api_key', image=>'buy_payssion.png',
         s_fields=>[
            {title=>'Your Payssion.com API key', name=>'payssion_api_key', type=>'text', comment=>'optional'},
            {title=>'Your Payssion.com Secret key', name=>'payssion_secret', type=>'text', comment=>'optional', size => '30'},
            ],
      };
}

sub checkout {
   my ($self, $f) = @_;
   return if $f->{type} ne 'payssion';
   my $ua = LWP::UserAgent->new;

   if(!$f->{method})
   {
      $ses->PrintTemplate("buy_payssion.html", %$f);
      exit;
   }

   my $sig = md5_hex(join('|',
       $c->{payssion_api_key},
       $f->{method},
       $f->{amount},
       $c->{currency_code},
       $f->{id},
       '', # sub_track_id
       $c->{payssion_secret}));
   
   my $res = $ua->post("https://www.payssion.com/api/v1/payment/create",
   {
      api_key => $c->{payssion_api_key},
      api_sig => $sig,
      pm_id => $f->{method},
      amount => $f->{amount},
      currency => $c->{currency_code},
      description => $c->{item_name},
      track_id => $f->{id},
      notify_url => "$c->{site_cgi}/ipn.cgi",
      success_url => "$c->{site_url}/?payment_complete=$f->{id}-1",
      redirect_url => "$c->{site_url}/?payment_complete=$f->{id}-1",
   });
   
   print STDERR "Received from Payssion: ", $res->decoded_content, "\n";
   my $ret = JSON::decode_json($res->decoded_content);

   die("Wrong result code") if $ret->{result_code} != 200;

   return($ret->{redirect_url});
}

sub verify {
   my ($self, $f) = @_;
   return if !$f->{pm_id};

   my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{track_id}) || die( "Transaction not found: '$f->{track_id}'"  );
   my $sig = md5_hex(join('|',
       $c->{payssion_api_key},
       $f->{pm_id},
       $f->{amount},
       $f->{currency},
       $f->{track_id},
       $f->{sub_track_id},
       $f->{state},
       $c->{payssion_secret}));

   print STDERR "sig=$sig\n";

   die("Wrong signature") if $f->{notify_sig} ne $sig;
	die("Wrong amount") if $f->{amount} != $transaction->{amount};
   die("Wrong currency") if $f->{currency} ne $c->{currency_code};
   die("Wrong state") if $f->{state} ne 'completed';

   return($transaction);
}

1;
