package Plugins::Payments::stripe;
use Payments;
use LWP::UserAgent;
use HTTP::Request::Common;
use base 'Payments';
use vars qw($ses $c);
use strict;

sub options {
	return {
		name=>'stripe',
		title=>'stripe.net',
		account_field=>'stripe_public',
		image=>'buy_stripe.png',
		listed_reseller=>'1',
		s_fields=>[
			{title=>'Your stripe.net Public Key', name=>'stripe_public', type=>'text', comment=>'optional', size=>36},
			{title=>'Your stripe.net Secret Key', name=>'stripe_secret', type=>'text', comment=>'optional', size=>36},
			]
		};
}

sub checkout {
	my ($self, $f) = @_;
   	return if $f->{type} ne 'stripe';
	$ses->PrintTemplate('buy_stripe.html',
		 stripe_public => $c->{stripe_public},
		 id => $f->{id},
		 amount => $f->{amount},
		 currency_code => $c->{currency_code});
	exit;
}

sub verify {
	my ($self, $f) = @_;
	return if !$f->{stripe_token};
	my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{tr_id}) || die("Transaction not found: '$f->{tr_id}'");

	my $req = POST "https://api.stripe.com/v1/charges", [
			   amount => int($transaction->{amount} * 100),
			   source => $f->{stripe_token},
			   currency => $c->{currency_code},
			   ];
	$req->authorization_basic($c->{stripe_secret}, '');

	my $res = LWP::UserAgent->new(agent => 'StripeAgent')->request($req);
	my $ret = JSON::decode_json($res->decoded_content);

	print STDERR "Stripe status: $ret->{status}\n";
	die("Wrong status") if $ret->{status} ne 'succeeded';

	return($transaction);
}

1;
