package Plugins::Payments::webmoney;
use Payments;
use base 'Payments';
use vars qw($ses $c);
use Digest::SHA qw(sha256_hex);

sub options {
	return {
		name=>'webmoney',
		title=>'Webmoney',
		account_field=>'webmoney_merchant_id',
		image=>'buy_webmoney.gif',
		listed_reseller=>'1',
			s_fields=>[
				{title=>'Your Webmoney Merchant ID', name=>'webmoney_merchant_id', type=>'text', comment=>'optional'},
				{title=>'Your Webmoney Secret Key', name=>'webmoney_secret_key', type=>'text', comment=>'optional'},
				],
		};
}

sub checkout
{
	my ($self, $f) = @_;
	return if $f->{type} ne 'webmoney';
	$ses->message("LMI_PAYMENT_NO out of bounds") if $f->{id} > 2**32;
	print"Content-type:text/html\n\n";
	print <<END
		<HTML><BODY onLoad="document.F1.submit();">
		<Form method="POST" action="https://merchant.wmtransfer.com/lmi/payment.asp" name="F1">
		<input type="hidden" name="LMI_PAYEE_PURSE" value="$c->{webmoney_merchant_id}">
		<input type="hidden" name="LMI_PAYMENT_AMOUNT" value="$f->{amount}">
		<input type="hidden" name="LMI_PAYMENT_DESC" value="$c->{item_name}">
		<input type="hidden" name="LMI_PAYMENT_NO" value="$f->{id}">
		<input type="hidden" name="LMI_RESULT_URL" value="$c->{site_cgi}/ipn.cgi">
		<input type="hidden" name="LMI_SUCCESS_URL" value="$c->{site_url}/?payment_complete=$f->{id}-0">
		<input type="hidden" name="LMI_FAIL_URL" value="$c->{site_url}">
		<input type="submit" value="Loading..." style="background:#fff; border:0;">
		</Form>
		</BODY></HTML>
END
		;
	exit;
}

sub verify {
	my ($self, $f) = @_;
	if($f->{LMI_PREREQUEST}) {
		print"Content-type:text/html\n\nYES";
		exit;
	}
	return if !$f->{LMI_PAYMENT_NO};
	my $transaction = $ses->db->SelectRow("SELECT * FROM Transactions WHERE id=?",$f->{LMI_PAYMENT_NO}) || die( "Transaction not found: '$f->{LMI_PAYMENT_NO}'"  );
	die("Amount changed!") unless $f->{LMI_PAYMENT_AMOUNT}==$transaction->{amount};
	die("Wrong wallet!") unless $f->{LMI_PAYEE_PURSE}==$c->{webmoney_merchant_id};
	if($f->{LMI_PREREQUEST})
	{
		print"Content-type:text/html\n\nYES";
		exit;
	}
	$f->{txn_id} = $f->{LMI_PAYER_PURSE};
#$ses->db->Exec("UPDATE Transactions SET txn_id=? WHERE id=?",$f->{txn_id}||'',$transaction->{id});

	my $hash_str = $f->{LMI_PAYEE_PURSE}.$f->{LMI_PAYMENT_AMOUNT}.$f->{LMI_PAYMENT_NO}.
		$f->{LMI_MODE}.$f->{LMI_SYS_INVS_NO}.$f->{LMI_SYS_TRANS_NO}.
		$f->{LMI_SYS_TRANS_DATE}.$c->{webmoney_secret_key}.$f->{LMI_PAYER_PURSE}.$f->{LMI_PAYER_WM};

	my $hash = uc sha256_hex($hash_str);
	die("SHA256 hash invalid!") unless $hash eq $f->{LMI_HASH};
	return($transaction);
}

1;
