#!/usr/bin/perl
use strict;
use lib '.';
use XFSConfig;
use CGI;
use CGI::Carp qw(fatalsToBrowser);
use JSON;
use Log;

my $log = Log->new(filename => 'api.log');

print("Access-Control-Allow-Origin: *\n");
print("Content-type: text/plain\n\nOK"), exit if $ENV{REQUEST_METHOD} eq 'OPTIONS';

$ENV{PERL_LWP_SSL_VERIFY_HOSTNAME} = 0;

my $q = CGI->new();
my $f;
$f->{$_}=$q->param($_) for $q->param;

$ENV{REMOTE_ADDR} = $ENV{HTTP_CF_CONNECTING_IP} if $c->{m_n_chunked_upload} && $ENV{HTTP_CF_CONNECTING_IP};

if($f->{op} !~ /^(compile|import_file)$/) {
   error( "OK:0:ERROR: unknown main server IP") if $c->{allowed_ip} && $ENV{REMOTE_ADDR} ne $c->{allowed_ip};
   error( "OK:0:ERROR: fs_key is wrong or not empty") if $f->{fs_key} ne $c->{fs_key};
   error( "OK:0:ERROR: you have to set fs_key first") if !$c->{fs_key} && $f->{op} !~ /^(test|update_conf)$/;
}

error("No op") if !$f->{op};

my $op_camelcase = ucfirst($f->{op});
$op_camelcase =~ s/\W//g;
$op_camelcase =~ s/_([a-z])/uc($1)/ge;

my $module = "Engine::FSRPC::$op_camelcase";
my $filename = "$c->{cgi_dir}/Engine/FSRPC/$op_camelcase.pm";

error("Unknown op: $f->{op}") if ! -e $filename;

require $filename;
my $ret = (\&{ "$module\::main" })->($f);

if(ref($ret)) {
   print "Content-type: application/json\n\n";
   print JSON::encode_json($ret);
} elsif(defined($ret)) {
   print "Content-type: text/html\n\n";
   print "$ret";
}

sub error {
   my ($msg) = @_;

   if($msg) {
      print "Status: 500\n";
      print STDERR "$msg\n";
   }

   print "Content-type: text/html\n\n";
   print "$msg\n";
   exit();
}
