#!/usr/bin/perl
use strict;
use lib '.';
use XFileConfig;
use Session;
use JSON;
use Log;

Log->new(filename => 'fs.log');

$ENV{PERL_LWP_SSL_VERIFY_HOSTNAME}=0;

$SIG{__WARN__} = sub {};

my $ses = Session->new;
my $f = $ses->f;
my $db = $ses->db;

if($f->{op} eq 'test')
{
   print"Content-type:text/html\n\nOK:".$ses->getIP;
   exit;
}

my $server = get_server();
error("No such file server") unless $server;
error("Wrong file server IP") unless $server->{srv_ip} eq $ses->getIP;

my $session = $db->SelectRow("SELECT * FROM Sessions WHERE session_id=?", $f->{sess_id});
my $torrent = $db->SelectRow("SELECT * FROM Torrents WHERE sid=?", $f->{sid});
my $user = get_user();

my $utype = $user ? ($user->{exp_sec}>0 ? 'prem' : 'reg') : 'anon';
$c->{$_}=$c->{"$_\_$utype"} for qw(disk_space max_upload_filesize max_rs_leech torrent_dl torrent_dl_slots remote_url torrent_seed_rate upload_on);

my $op_camelcase = ucfirst($f->{op} || 'save_file');
$op_camelcase =~ s/\W//g;
$op_camelcase =~ s/_([a-z])/uc($1)/ge;

my $module = $ses->require("Engine::MainRPC::$op_camelcase");
my $ret = (\&{ "$module\::main" })->({ server => $server, user => $user, session => $session });

if(ref($ret)) {
   print "Content-type: application/json\n\n";
   print JSON::encode_json($ret);
} elsif(defined($ret)) {
   print "Content-type: text/html\n\n";
   print $ret;
}

sub error {
   my ($msg) = @_;

   if($msg) {
      print "Status: 500\n";
      print STDERR "$msg\n";
   }

   print "Content-type: text/html\n\n";
   print "$msg\n";
   exit();
}

sub get_server {
   return $db->SelectRow("SELECT * FROM Servers WHERE srv_key=?", $f->{fs_key} );
}

sub get_user {
   my $ur = $ses->require("UsersRegistry");
   return $ur->getUserById($torrent->{usr_id}) if $torrent;
   return $ur->getUserById($session->{usr_id}) if $session && $session->{usr_id};
   return $ur->getUserById($f->{usr_id}) if $f->{usr_id};
   return $ur->getUserById($db->SelectOne("SELECT usr_id FROM Users WHERE usr_login=?", $f->{usr_login})) if $f->{usr_login};
   return $ses->require("Auth")->checkLoginPass( $f->{check_login}, $f->{check_pass}) || error("Invalid login / pass") if $f->{check_login};
}
