#!/usr/bin/perl
use strict;
use lib '.';
use XFSConfig;
use MultiHDD;
use Log;
use JSON;

my $log = Log->new(filename => 'put_chunk.log');

print("Access-Control-Allow-Origin: *\n");
print("Access-Control-Allow-Methods: PUT\n");
print("Access-Control-Allow-Headers: Content-Type,X-Upload-SID\n");
print("Content-type: text/plain\n\n"), exit if $ENV{REQUEST_METHOD} ne 'PUT';

my $sid = $ENV{HTTP_X_UPLOAD_SID};
$sid =~ s/\W//g;
&Return(500, "No SID specified") if !$sid;

print STDERR "Starting upload: $sid\n";
my $disk_id = MultiHDD::locate($c->{temp_dir}, "$sid");

unless($disk_id) {
    $disk_id = MultiHDD::selectUploadDisk($c->{upload_dir});
}
print STDERR "Using Disk ID: $disk_id\n";

open FH, ">>$c->{temp_dir}/$disk_id/$sid" || &Return(500, "Couldn't open file: $!");
binmode(FH);
binmode(STDIN);

my ($buf, $bytes_read);

while(read(STDIN, $buf, 1024 * 64))
{
   print FH $buf;
   $bytes_read += length($buf);
}

close(FH);

print STDERR "Upload ok\n";

Return($bytes_read ? 200 : 500, JSON::encode_json({ status => 'OK' }));

sub Return
{
   my ($code, $msg) = @_;
   print "Status: $code\n";
   print "Content-type: text/plain\n\n";
   print "$msg\n";
   exit();
}
