#!/usr/bin/perl -X
use strict;
use lib '.';
use XFSConfig;
use Fcntl qw( :flock :DEFAULT );
use LWP::UserAgent;
use XUpload;
use JSON;
use Log;

my $log = Log->new(filename=>'url_upload.log');
my $ua = LWP::UserAgent->new(agent => 'Mozilla/5.0 (Windows; U; Windows NT 5.1; ru; rv:1.9.1.6) Gecko/20091201 Firefox/3.5.6 (.NET CLR 3.5.30729)', timeout => 10);

for(1..10000) {
   sleep(1);
   my $res = $ua->post("$c->{site_cgi}/fs.cgi", { op => "queue_url_next", fs_key => $c->{fs_key} });
   print $res->decoded_content, "\n";

   my $task = eval { JSON::decode_json($res->decoded_content) } if length($res->decoded_content);
   print STDERR "Failed while decoding JSON: ", $res->decoded_content, "\n" if $@;
   next if !$task;

   my $tempfile = "$c->{temp_dir}/".join('', map int rand(10), 1..10);
   print STDERR "tempfile = $tempfile\n";

   open FILE, ">$tempfile"|| die"Can't open dest file:$!";
   my $req = HTTP::Request->new(GET => $task->{url});

   my $progress = { time => time() };

   my $resp = $ua->request($req, sub {
      my ($buffer, $res) = @_;
      print FILE $buffer;
      $progress->{loaded} += length($buffer);

      my $dt = 1;
      my $time = time();

      if($time - $progress->{time} > $dt) {
         $progress->{total_size} ||= int($res->content_length());
         $progress->{time} = time();
         $progress->{speed} = ($progress->{loaded} - $progress->{loaded_prev}) / $dt;
         $progress->{loaded_prev} = $progress->{loaded};
         print JSON->new->canonical->encode($progress), "\n";

         print "Pushing progress to the main server\n";

         my $res = ua()->post("$c->{site_cgi}/fs.cgi", {
            op          => "queue_url_progress",
            fs_key      => $c->{fs_key},
            id          => $task->{id},
            size_full   => $progress->{total_size},
            size_dl     => $progress->{loaded},
            speed       => $progress->{speed},
         });

         print "Got result: ", $res->decoded_content, "\n";
      }
   });

   close FILE;

   my $file = XUpload::ProcessFile({ file_tmp => $tempfile, file_name_orig => compute_name($resp), file_size => -s $tempfile, usr_id => $task->{usr_id} });
   print JSON::encode_json($file), "\n";

   print "Marking URL upload task as completed\n";

   my $res = $ua->post("$c->{site_cgi}/fs.cgi", {
      op => "queue_url_done",
      fs_key => $c->{fs_key},
      id => $task->{id},
   });
   
   print "Result: ", $res->decoded_content(), "\n";
}

sub acquire_slot {
   my ($name, %opts) = @_;

   for(1..$opts{max_process_count}) {
      return 1 if sysopen( PID, "lockfiles/$name$_.lock", O_RDWR | O_CREAT ) && flock( PID, LOCK_EX | LOCK_NB );
   }
}

sub compute_name {
   my ($resp) = @_;
   my $content_disp = $resp->header('Content-Disposition');
   return $1 if $content_disp =~ /filename="(.+)"/
      || $content_disp =~ /filename=(.+)/
      || $content_disp =~ /filename\*=.*?''(.+)/;
   return $1 if URI->new($resp->request->uri)->path =~ /([^\/]+)$/;
}

sub ua {
   return LWP::UserAgent->new(agent => 'XFS-FSAgent');
}
