#!/usr/bin/perl
use strict;
use lib ".";
use XFSConfig;
use MultiHDD;
use CGI::Carp qw(fatalsToBrowser);
use MIME::Base64;
use URI::Escape qw(uri_unescape);
use File::Basename;
use Data::Dumper;
use XUpload;
use Log;

Log->new(filename => 'webdav_put.log');

my $sess_id = $1 if $ENV{QUERY_STRING} =~ /sess_id=(\w+)/;
&Return(501, "Not implemented") if $ENV{REQUEST_METHOD} !~ /^(PUT|HEAD)$/;
&Return(501, "No sess_id") if !$sess_id;

sub randchar
{ 
   my @range = ('0'..'9','a'..'z');
   my $x = int scalar @range;
   join '', map $range[rand $x], 1..shift||1;
}

sub readToFile
{
   my ($filename) = @_;

   my $bytes_read = 0;

   open F, ">$filename";
   binmode(F);

   my $buf;
   while(read(STDIN, $buf, 1024 * 64))
   {
      print F $buf;
      $bytes_read += length($buf);
   }

   close F;

   return $bytes_read;
}

my $disk_id = MultiHDD::selectUploadDisk($c->{upload_dir});
my $path = uri_unescape($ENV{PATH_INFO});
my $temp_file = "$c->{temp_dir}/$disk_id/" . randchar(16);
&Return(404, "Invalid path") if !$path;

binmode(STDIN);
my $bytes_read = readToFile($temp_file);

if($bytes_read > 0)
{
   my $xfile = { file_tmp => $temp_file, file_name_orig => basename($path), file_public => 1 };
   $xfile->{disk_id} = $disk_id;
   
   my $file = XUpload::ProcessFile($xfile, {
      ip => $ENV{REMOTE_ADDR},
      fld_path => dirname($path),
      sess_id => $sess_id,
      file_upload_method => 'dav' });
   
   &Return(500, $file->{msg}) if $file->{msg} ne 'OK';
}

print "Status: 204\n\n";

sub Return
{
   my ($code, $msg) = @_;
   print "Status: $code\n";
   print "Content-type: text/plain\n\n";
   print "$msg\n";
   exit();
}
