#!/usr/bin/perl
use strict;
use XFileConfig;
use CGI::Carp qw(fatalsToBrowser);
use Session;
use XUtils;

my $ses = Session->new();
my $db = $ses->db;
my $f = $ses->f;

# Status Codes:
# 200: Everything is OK. Request succeeded
# 400: Bad request (e.g. wrong parameters)
# 403: Permission denied (wrong api login/key, action on a file which does not belong to you, ...)
# 404: File not found
# 451: Unavailable For Legal Reasons

out({"status"=>403, msg=>"Not enabled"}) if !$c->{m_7};

my ($usr_id,$api_key) = $f->{key}=~/^(\d+)(\w{16})$/;
out({"status"=>400, msg=>"Invalid key"}) unless $usr_id && $api_key;

my $user = $db->SelectRow("SELECT *, UNIX_TIMESTAMP(usr_premium_expire)-UNIX_TIMESTAMP() as exp_sec FROM Users WHERE usr_id=? AND usr_api_key=?",$usr_id,$api_key);
sleep(1),out({"status"=>403, msg=>"Wrong auth"}) unless $user;
$user->{premium}=1 if $user->{exp_sec}>0;

$f->{op} = "$1_$2" if $ENV{REQUEST_URI}=~/api\/(\w+)\/(\w+)/i;

my $sub={
	account_info	=> \&AccountInfo,
	account_stats	=> \&AccountStats,
	upload_url		=> \&UploadURL,
	upload_server	=> \&UploadServer,
	file_clone 		=> \&Clone,
	file_info 		=> \&FileInfo,
	file_rename 	=> \&FileRename,
	file_direct_link=> \&FileDirectLink,
	file_set_folder => \&FileSetFolder,
	file_list		=> \&FileList,
	folder_list 	=> \&FolderList,
	folder_rename	=> \&FolderRename,
	folder_create 	=> \&FolderCreate,
	dmca_list		=> \&DMCAList,
	encodings 		=> \&Encodings,
	adm_delete 		=> \&AdminDelete,
	files_deleted	=> \&FilesDeleted,
}->{ $f->{op} };

if($sub)
{
	&$sub
}
else
{
	out({"status"=>400, msg=>"Invalid operation"});
}

sub UploadURL
{
   my $code = $ses->randchar(12);
   while($db->SelectOne("SELECT file_id FROM Files WHERE file_code=? OR file_real=?",$code,$code))
   {
	$code = $ses->randchar(12);
	while($db->SelectOne("SELECT id FROM QueueUpload WHERE file_code=?",$code)){ $code = $ses->randchar(12); }
   }

   my $folder = $db->SelectRow("SELECT * FROM Folders WHERE usr_id=? AND fld_id=?", $user->{usr_id}, $f->{folder}) if $f->{folder}=~/^\d+$/;
   my $fld_id = $folder ? $folder->{fld_id} : 0;

   $db->Exec("INSERT INTO QueueUpload SET usr_id=?, url=?, file_code=?, fld_id=?, created=NOW()", $user->{usr_id}, $f->{url}, $code, $fld_id );
   out({"status"=>200, "msg"=>"OK", result=>{"filecode"=>$code} });
}

sub AdminDelete
{
    out({"status"=>403, msg=>"Access denied"}) unless $user->{usr_adm};
    my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=?",$f->{file_code});
    out({"status"=>404, msg=>"No file"}) unless $file;
    $ses->DeleteFile($file);
    out({"status"=>200, msg => "OK"});
}

sub Clone
{
    out({"status"=>403, "msg"=>"This function not allowed in API"}) unless $c->{m_7_clone};
    my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=?",$f->{file_code});
    out({"status"=>404, "msg"=>"No file"}) unless $file;
    my $code = CloneFile($file, fld_id => $f->{fld_id});
    out({ "status"=>200, "msg"=>"OK", "result"=>{"filecode"=>$code,"url"=>"$c->{site_url}/$code"} });
}

sub DMCAList
{
   my $list = $db->SelectARef("SELECT * FROM FilesDMCA d, Files f WHERE d.usr_id=? AND d.file_id=f.file_id ORDER BY del_time", $user->{usr_id});
   my @arr;
   for(@$list)
   {
     push @arr, { file_code=>$_->{file_code}, del_time=>$_->{del_time} };
   }
   out({"status"=>200, "msg"=>"OK", result => \@arr});
}

###########

sub out
{
   my $data = shift;
   $data->{server_time} = sprintf("%d-%02d-%02d %02d:%02d:%02d", $ses->getTime() );
   $ses->PrintJSON($data);
   exit;
}

sub CloneFile
{
   my ($file,%opts) = @_;
   my $db = $ses->db;

   my $code = $ses->randchar(12);
   while($db->SelectOne("SELECT file_id FROM Files WHERE file_code=? OR file_real=?",$code,$code)){$code = $ses->randchar(12);}

   $db->Exec("INSERT INTO Files 
        SET usr_id=?, 
            srv_id=?,
            file_fld_id=?,
            file_name=?, 
            file_descr=?, 
            file_public=?, 
            file_code=?, 
            file_real=?, 
            file_real_id=?, 
            file_del_id=?, 
            file_size=?,
            file_size_encoded=?,
            file_password=?, 
            file_ip=?, 
            file_md5=?, 
            file_spec=?, 
            file_upload_method='copy',
            file_created=NOW(), 
            file_last_download=NOW()",
         $opts{usr_id}||$user->{usr_id},
         $file->{srv_id},
         $opts{fld_id}||0,
         $file->{file_name},
         '',
         1,
         $code,
         $file->{file_real},
         $file->{file_real_id}||$file->{file_id},
         $file->{file_del_id},
         $file->{file_size},
         $file->{file_size_encoded},
         $opts{file_password}||'',
         $opts{ip}||$ses->getIP,
         $file->{file_md5},
         $file->{file_spec}||'',
       );
   $db->Exec("UPDATE Servers SET srv_files=srv_files+1 WHERE srv_id=?",$file->{srv_id});
   return $code;
}

sub FileInfo
{
  my @list = grep{/^\w{12}$/} split /,/, $f->{file_code};
  out({"status"=>400, "msg"=>"Invalid file codes"}) unless @list;
  @list = splice @list, 0, 50;
  my $codes = join ',', map{qq['$_']} @list;
  my $list = $db->SelectARef("SELECT f.*
				FROM Files f
				WHERE file_code IN ($codes) 
				LIMIT 100");
  my $hh;
  $hh->{$_->{file_code}}=$_ for @$list;
  my @arr;
  for(@list)
  {
    if($hh->{$_})
    {
      my $status = 200;
      $status=451 if $hh->{$_}->{del_time};
      push @arr, {
         status => $status, 
         filecode =>$_, 
         name =>$hh->{$_}->{file_name},
         downloads => $hh->{$_}->{file_downloads},
         uploaded => $hh->{$_}->{file_created},
         size =>$hh->{$_}->{file_size},
      };
    }
    else
    {
      push @arr, {status=>404, filecode=>$_};
    }
  }
  out({"status"=>200, "msg"=>"OK", result => \@arr});
}

sub FolderList
{
  my $folder = $db->SelectRow("SELECT * FROM Folders WHERE fld_id=? AND usr_id=?", $f->{fld_id}, $user->{usr_id} ) if $f->{fld_id};
  out({"status"=>403, "msg"=>"Folder not exist or not yours"}) if !$folder && $f->{fld_id};
  $folder->{fld_id}=0 unless $folder;
  my $fld_list = $db->SelectARef("SELECT * FROM Folders WHERE usr_id=? AND fld_parent_id=?", $user->{usr_id}, $folder->{fld_id});
  my $list = $db->SelectARef("SELECT * FROM Files WHERE usr_id=? AND file_fld_id=? ORDER BY file_created DESC", $user->{usr_id}, $folder->{fld_id});

  my @folders;
  for(@$fld_list)
  {
    push @folders, {fld_id=>$_->{fld_id}, name=>$_->{fld_name}, code=>$_->{fld_code}};
  }

  my @files;
  for(@$list)
  {
    push @files, {  file_code=>$_->{file_code}, 
    				fld_id=>$_->{file_fld_id}, 
    				name=>$_->{file_name}, 
    				uploaded=>$_->{file_created}, 
    				link=>"$c->{site_url}/$_->{file_code}.html", 
    			};
  }

  out({"status"=>200, "msg"=>"OK", result => {folders=>\@folders, files=>\@files} });
}

sub FolderRename
{
   out({"status"=>400, msg=>"Folder id and name required"}) unless $f->{fld_id}=~/^\d+$/ && $f->{name};
   my $folder = $db->SelectRow("SELECT * FROM Folders WHERE fld_id=? AND usr_id=?", $f->{fld_id}, $user->{usr_id} );
   out({"status"=>403, "msg"=>"Folder not exist or not yours"}) unless $folder;
   $f->{name} = $ses->SecureStr($f->{name});
   out({"status"=>400, msg=>"Invalid folder name"}) unless $f->{name} && length($f->{name})>=2;
   $db->Exec("UPDATE Folders SET fld_name=? WHERE fld_id=? LIMIT 1", $f->{name}, $folder->{fld_id});

   out({"status"=>200, "msg"=>"OK", result=>'true' });
}

sub FileRename
{
   out({"status"=>400, msg=>"File code and name required"}) unless $f->{file_code}=~/^\w{12}$/ && $f->{name};
   my $file = $db->SelectRow("SELECT * FROM Files WHERE file_code=? AND usr_id=?", $f->{file_code}, $user->{usr_id} );
   out({"status"=>403, "msg"=>"File not exist or not yours"}) unless $file;
   $f->{name} = $ses->SecureStr($f->{name});
   out({"status"=>400, msg=>"Invalid file name"}) unless $f->{name} && length($f->{name})>=2;
   $db->Exec("UPDATE Files SET file_name=? WHERE file_id=? LIMIT 1", $f->{name}, $file->{file_id});

   out({"status"=>200, "msg"=>"OK", result=>'true' });
}

sub Encodings
{
   my $list = $db->SelectARef("SELECT *, UNIX_TIMESTAMP()-UNIX_TIMESTAMP(q.updated) as updated2 
				FROM QueueEncoding q, Files f 
				WHERE f.usr_id=?
				AND q.file_id=f.file_id
				GROUP BY q.file_id
				ORDER BY status DESC, started DESC", $user->{usr_id});
   my @arr;
   for(@$list)
   {
     $_->{status}='STUCK' if $_->{status} eq 'ENCODING' && $_->{updated2} > 180;
     push @arr, { name=>$_->{file_title}, status=>$_->{status}, progress=>int($_->{progress}/100), link=>$ses->makeFileLink($_) };
   }

   out({"status"=>200, "msg"=>"OK", result=>\@arr });
}

sub FileDirectLink
{
   out({"status"=>403, "msg"=>"This function not allowed in API"}) unless $c->{m_7_direct};
   my $file = $db->SelectRow("SELECT * FROM Files f
      LEFT JOIN Servers s ON s.srv_id=f.srv_id
      WHERE file_code=?
      AND usr_id=?",
      $f->{file_code},
      $user->{usr_id});

   out({"status"=>404,"msg"=>"no file"}) if !$file;
   out({"status"=>200, "msg"=>"OK", result => { size => $file->{file_size}, url => $ses->getPlugins('CDN')->genDirectLink($file) } });
}

sub AccountInfo
{
	my $utype = $user->{exp_sec} > 0 ? 'prem' : 'reg';
	my $disk_space = $user->{usr_disk_space} || $c->{"disk_space_$utype"};
	my $totals = $db->SelectRow( "SELECT COUNT(*) as total_files, SUM(file_size) as total_size FROM Files WHERE usr_id=?", $user->{usr_id} );

	my $data = {
					email	=> $user->{usr_email},
					balance	=> $user->{usr_money},
					premium_expire => $user->{usr_premium_expire},
					storage_used => $totals->{total_size},
					storage_left => $disk_space ? ($disk_space * 2**20 - $totals->{total_size}) : 'inf',
				};

	out({"status"=>200, "msg"=>"OK", result=>$data });
}

sub FolderCreate
{
   my $fld_parent_id = $db->SelectOne("SELECT fld_id FROM Folders WHERE fld_id=? AND usr_id=?", $f->{parent_id}, $user->{usr_id} ) if $f->{parent_id};
   out({"status"=>403, "msg"=>"Parent folder not exist or not yours"}) if $f->{parent_id} && !$fld_parent_id;
   $f->{name} = $ses->SecureStr($f->{name});
   out({"status"=>400, msg=>"Invalid folder name"}) unless $f->{name} && length($f->{name})>=2;

   $db->Exec( "INSERT INTO Folders SET usr_id=?, fld_parent_id=?, fld_name=?",
      $user->{usr_id}, $fld_parent_id||0, $f->{name} );

   out({"status"=>200, "msg"=>"OK", result=>{fld_id => $db->getLastInsertId} });
}

sub FilesDeleted
{
	$f->{last} = 24*60 unless $f->{last}=~/^\d+$/;
	my $files = $db->SelectARef("SELECT f.*, UNIX_TIMESTAMP()-UNIX_TIMESTAMP(deleted) as ago
                                 FROM FilesDeleted f 
                                 WHERE f.usr_id=?
                                 AND hide=0
                                 AND deleted > NOW()-INTERVAL ? MINUTE
                                 ORDER BY deleted DESC",$user->{usr_id},$f->{last});
	my @list;
	for(@$files)
	{
		push @list, {file_code => $_->{file_code},
						name => $_->{file_name},
						deleted => $_->{deleted},
						deleted_ago_sec => $_->{ago},
					};
	}
	out({"status"=>200, "msg"=>"OK", result=>\@list });
}

sub FileList
{
  my $folder = $db->SelectRow("SELECT * FROM Folders WHERE fld_id=? AND usr_id=?", $f->{fld_id}, $user->{usr_id} ) if $f->{fld_id};
  my @filters;
  push @filters,"file_fld_id=$folder->{fld_id}" if $folder;
  push @filters,"file_public=$f->{public}" if $f->{public}=~/^0|1$/;
  push @filters,"file_created>'$f->{created}'" if $f->{created}=~/^\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d$/;
  push @filters,"file_created>NOW()-INTERVAL $f->{created} MINUTE" if $f->{created}=~/^\d+$/;
  push @filters,"file_name LIKE '%$f->{name}%'" if $f->{name} && $f->{name}!~/[\"\0\;\n\r]+/;
  my $filter_str = ' AND '.join(' AND ', @filters) if @filters;
  $f->{page}=~s/\D+//;
  $f->{per_page}=~s/\D+//;
  $f->{per_page}=1000 if $f->{per_page}>1000;
  $f->{per_page}||=100;
  my $list = $db->SelectARef("SELECT * FROM Files 
  								WHERE usr_id=? 
  								$filter_str 
  								ORDER BY file_created DESC".$ses->makePagingSQLSuffix($f->{page},$f->{per_page}), $user->{usr_id});
  my $total = $db->SelectOne("SELECT COUNT(*) FROM Files 
  								WHERE usr_id=? 
  								$filter_str 
  								", $user->{usr_id});

  my @files;
  for(@$list)
  {
    $ses->getThumbLink($_);
    push @files, {  file_code=>$_->{file_code}, 
    				fld_id=>$_->{file_fld_id}, 
    				name=>$_->{file_name}, 
    				uploaded=>$_->{file_created}, 
    				size => $_->{file_size},
    				public => $_->{file_public},
    				downloads => $_->{file_downloads},
    				link => $ses->makeFileLink($_), 
    				thumbnail => $_->{video_thumb_url},
    			};
  }

  out({"status"=>200, "msg"=>"OK", result => {results => scalar(@files), results_total => $total, files=>\@files} });
}

sub UploadServer
{
   my $server = $ses->require("Engine::Components::ServersRegistry")->selectServer($user);
	out({"status"=>200, "msg"=>"No servers available for uploads", result=>{} }) if !$server;

   my $sess_id = $user ? $ses->require("Engine::Components::SessionTracker")->GetSession($user->{usr_id}) : undef;
	out({"status"=>200, "msg"=>"OK", result=>"$server->{srv_cgi_url}/upload.cgi", sess_id => $sess_id });
}

sub AccountStats
{
   $f->{last}=~s/\D+//g;
   $f->{last}||=7;
   my @d1 = $ses->getTime(time-$f->{last}*24*3600);
   $d1[2]='01';
   my @d2 = $ses->getTime();
   my $day1 = $f->{date1}=~/^\d\d\d\d-\d\d-\d\d$/ ? $f->{date1} : "$d1[0]-$d1[1]-$d1[2]";
   my $day2 = $f->{date2}=~/^\d\d\d\d-\d\d-\d\d$/ ? $f->{date2} : "$d2[0]-$d2[1]-$d2[2]";
   my $list2 = $db->SelectARef("SELECT *, DATE_FORMAT(day,'%e') as day2
                                     FROM Stats2
                                     WHERE usr_id=?
                                     AND day>=?
                                     AND  day<=?
                                     ORDER BY day",$user->{usr_id},$day1,$day2);
   out({"status"=>200, "msg"=>"Not enough reports data", result=>{} }) if $#$list2<0;
   my @arr;
   for my $x (@$list2)
   {
   		my $y={};
   		$y->{profit_total} = sprintf("%.05f",$x->{profit_dl}+$x->{profit_sales}+$x->{profit_rebills}+$x->{profit_refs}+$x->{profit_site});
   		$y->{$_}=$x->{$_} for qw(day downloads sales profit_dl profit_sales profit_rebills profit_refs profit_site);
   		push @arr, $y;
   }
   out({"status"=>200, "msg"=>"OK", result=>\@arr });
}

sub FileSetFolder
{
	my @codes = grep{/^\w{12}$/} split(/\s*,\s*/,$f->{file_code});
	my $ids = join "','", @codes;
	out({"status"=>400, "msg"=>"Invalid file code"}) unless $ids;
	out({"status"=>400, "msg"=>"Invalid folder id"}) unless $f->{fld_id}=~/^\d+$/;
	out({"status"=>403, "msg"=>"Your account don't have such folder"}) unless $db->SelectOne("SELECT fld_id FROM Folders WHERE usr_id=? AND fld_id=?",$user->{usr_id},$f->{fld_id});
	$db->Exec("UPDATE Files SET file_fld_id=? WHERE usr_id=? AND file_code IN ('$ids')", $f->{fld_id}, $user->{usr_id});
	out({"status"=>200, "msg"=>"OK"});
}
