package Engine::FSRPC::Test;
use strict;

use XFSConfig;
use LWP::UserAgent;
use MultiHDD;

sub main
{
   my ($f) = @_;
   my @tests;
   # Try to CHMOD first
   chmod 0777, $c->{temp_dir};
   chmod 0777, $c->{upload_dir};
   chmod 0777, $c->{htdocs_dir};
   chmod 0755, 'upload.cgi';
   chmod 0755, 'upload_status.cgi';
   chmod 0666, 'XFSConfig.pm';
   chmod 0666, 'logs.txt';

   # temp dir
   push @tests, -d $c->{temp_dir} ? 'temp dir exist: OK' : "temp dir exist: ERROR($!)";
   push @tests, mkdir("$c->{temp_dir}/test") ? 'temp dir mkdir: OK' : "temp dir mkdir: ERROR($!)";
   push @tests, rmdir("$c->{temp_dir}/test") ? 'temp dir rmdir: OK' : "temp dir rmdir: ERROR($!)";

   # url temp dir
   push @tests, -d $c->{htdocs_tmp_dir} ? 'tmp dir exist: OK' : "tmp dir exist: ERROR($!)";
   push @tests, mkdir("$c->{htdocs_tmp_dir}/test") ? 'tmp dir mkdir: OK' : "tmp dir mkdir: ERROR($!)";
   push @tests, rmdir("$c->{htdocs_tmp_dir}/test") ? 'tmp dir rmdir: OK' : "tmp dir rmdir: ERROR($!)";

   # upload dir
   push @tests, -d $c->{upload_dir} ? 'upload dir exist: OK' : "upload dir exist: ERROR($!)";
   push @tests, mkdir("$c->{upload_dir}/test") ? 'upload dir mkdir: OK' : "upload dir mkdir: ERROR($!)";
   push @tests, rmdir("$c->{upload_dir}/test") ? 'upload dir rmdir: OK' : "upload dir rmdir: ERROR($!)";

   for(MultiHDD::disks($c->{upload_dir})) {
      push @tests, mkdir("$c->{temp_dir}/$_/test") ? "disk$_ temp dir mkdir: OK" : "disk$_ temp dir mkdir: ERROR($!)";
      push @tests, rmdir("$c->{temp_dir}/$_/test") ? "disk$_ temp dir rmdir: OK" : "disk$_ temp dir rmdir: ERROR($!)";
      push @tests, mkdir("$c->{upload_dir}/$_/test") ? "disk$_ upload dir mkdir: OK" : "disk$_ upload dir mkdir: ERROR($!)";
      push @tests, rmdir("$c->{upload_dir}/$_/test") ? "disk$_ upload dir rmdir: OK" : "disk$_ upload dir rmdir: ERROR($!)";

      push @tests, _try_write_file("$c->{temp_dir}/$_/testfile") ? "disk$_ write file: OK" : "disk$_ write file: ERROR($!)";
      push @tests, rename("$c->{temp_dir}/$_/testfile", "$c->{upload_dir}/$_/testfile") ? "disk$_ move file: OK" : "disk$_ move file: ERROR($!)";
      push @tests, unlink("$c->{upload_dir}/$_/testfile") ? "disk$_ remove file: OK" : "disk$_ remove file: ERROR($!)";
   }

   if($f->{srv_ftp}) {
      my $host = $1 if $f->{srv_cgi_url} =~ /^https?:\/\/([^\/:]+)/;
      my $port = 2121;
      push @tests, _tcp_connect($host, $port) ? "ftp-slave connect: OK" : "ftp-slave connect: ERROR ($!)";
   }

   # htdocs dir
   push @tests, -d $c->{htdocs_dir} ? 'htdocs dir exist: OK' : "htdocs dir exist: ERROR($!)";
   push @tests, mkdir("$c->{htdocs_dir}/test") ? 'htdocs dir mkdir: OK' : "htdocs dir mkdir: ERROR($!)";
   push @tests, symlink("upload.cgi","$c->{htdocs_dir}/test/test.avi") ? 'htdocs dir symlink: OK' : "htdocs dir symlink: ERROR($!)";
   push @tests, unlink("$c->{htdocs_dir}/test/test.avi") ? 'htdocs dir symlink del: OK' : "htdocs dir symlink del: ERROR($!)";
   push @tests, rmdir("$c->{htdocs_dir}/test") ? 'htdocs dir rmdir: OK' : "htdocs dir rmdir: ERROR($!)";
   # XFSConfig.pm
   push @tests, open(F,'XFSConfig.pm') ? 'config read: OK' : "config read: ERROR($!)";
   push @tests, open(F,'>>XFSConfig.pm') ? 'config write: OK' : "config write: ERROR($!)";

   my $site_cgi = $f->{site_cgi};
   my $ua = LWP::UserAgent->new(agent => "XFS-FSAgent",timeout => 90);
   my $res = $ua->post("$site_cgi/fs.cgi",
                       {
                          op => 'test'
                       }
                      );
   push @tests, $res->content =~ /^OK/ ? 'fs.cgi: OK' : 'fs.cgi: ERROR '.$res->content;
   my ($ip) = $res->content =~ /^OK:(.*)/;
   
   return  "OK:$ip:".join('|',@tests) ;
}

sub _try_write_file {
   my ($fname) = @_;
   open(my $file, ">$fname") || return;
   print $file "\0" x 16;
   close($file);

   return 1;
}

sub _tcp_connect {
   my ($host, $port) = @_;
   use IO::Socket;
   return IO::Socket::INET->new(PeerAddr => $host,
                                PeerPort => $port,
                                Proto    => "tcp",
                                Type     => IO::Socket::INET::SOCK_STREAM);
}

1;
