package MultiHDD;
use strict;
use Exporter;
use SafeRun;
use List::Util qw(shuffle);

@MultiHDD::ISA = qw(Exporter);
@MultiHDD::EXPORT = qw(disks locate selectUploadDisk diskUsage);

sub disks {
   my ($pool) = @_;
   die("Pool not specified") if !$pool;

   opendir(DIR, $pool);
   my @ret = grep { /^\d{2}$/ } readdir(DIR);
   closedir(DIR);

   return sort(@ret);
}

sub locate {
   my ($pool, $relpath) = @_;
   my ($disk_id) = grep { -e "$pool/$_/$relpath" } disks($pool);
   return $disk_id;
}

sub selectUploadDisk {
   my ($pool) = @_;

   my @disks = disks($pool);
   return if !@disks; # Fallback on single-disk configurations

   my ($most_free) = sort { $b->{available} <=> $a->{available} } shuffle(diskUsage($pool));
   return $most_free->{disk_id} if $most_free;
}

sub diskUsage {
   my ($pool) = @_;
   my $ret = {};

   my @disks = disks($pool);
   my @stdout = saferun("df", @disks ? map { "$pool/$_" } @disks : $pool)->stdout;
   my @lines = splice(@stdout, 1);

   for(@lines) {
      my $disk = shift(@disks);
      my @values = split(/\s+/, $_);
      $values[$_] *= 1024 for (1..3);

      $ret->{$disk}->{disk_id} = $disk;
      ($ret->{$disk}->{device}, $ret->{$disk}->{total}, $ret->{$disk}->{used}, $ret->{$disk}->{available}) = @values;
   }

   return wantarray() ? values(%$ret) : $ret;
}

1;
