#!/usr/bin/perl

use strict;
$|++;

BEGIN
{
   use File::Basename qw 'dirname';
   chdir(dirname($0));
}

use lib '.';
use XFSConfig;
use Getopt::Long;
use Cwd qw(realpath);
use JSON;

use XUpload;

print_help_and_exit() if !@ARGV;

my %opts;
my $result = GetOptions(\%opts, "basedir=s", "upload_method=s");
print_help_and_exit() if !$result;
print_help_and_exit() if !$opts{basedir};

die("Directory not exists: $opts{basedir}") if ! -d $opts{basedir};

while(my $line = <STDIN>)
{
   chomp $line;
   my $object = eval { JSON::decode_json($line) };

   error($@),next if $@;
   error("Can't go below $opts{basedir}"),next if !_is_subpath_of($object->{file_tmp}, $opts{basedir});

   my $xfile = _filter($object, qw(file_tmp file_name_orig ));
   my $f = _filter($object, qw(ip fld_path usr_login));

   $f->{file_upload_method} = $opts{upload_method} || '';

   my $file = eval { XUpload::ProcessFile($xfile, $f) };
   error($@),next if $@;
   error($file->{file_status}),next if $file->{file_status};
   print JSON::encode_json($file), "\n";
}

sub print_help_and_exit
{
   print STDERR "Usage: $0 --basedir <base_dir> [--upload_method <upload_method>]\n";
   exit(-1);
}

sub error
{
   my ($msg) = @_;
   print JSON::encode_json({ status => 'ERROR', msg => $msg }), "\n";
}

sub _filter
{
   my ($hashref, @fields) = @_;
   return { map { $_ => $hashref->{$_} } @fields }
}

sub _is_subpath_of
{
   my ($path, $basepath) = @_;
   my @path = _splitpath(realpath($path));
   my @basepath = _splitpath($basepath);

   for(my $i = 0; $i < @basepath; $i++)
   {
      return 0 if $basepath[$i] ne $path[$i];
   }

   return 1;
}

sub _splitpath
{
   my ($path) = @_;
   $path =~ s/^\/+//;
   $path =~ s/\/+$//;
   return split(/\/+/, $path);
}
